@@ -4,21 +4,17 @@
  *	Response handling.
  */
 
+#include <stdlib.h>
 #include "config.h"
 #include "news.h"
 #include "keymap.h"
 #include "nn_term.h"
 #include "options.h"
 #include "regexp.h"
-#include "chset.h"
+#include "aux.h"
 
 /* answer.c */
 
-#ifdef NNTP
-# ifdef NNTP_MINI_INEWS_HEADER
-static mini_inews_header __APROTO((FILE *t));
-# endif
-#endif
 static subj_line __APROTO((FILE *t, int re, char *subj, char *prefix));
 static void ng_line __APROTO((FILE *t, int use_follow));
 static void ref_line __APROTO((FILE *t));
@@ -26,10 +22,6 @@ static to_line __APROTO((FILE *t, char *to));
 static void alt_to_line __APROTO((FILE *t, char *to, int mask));
 static void end_header __APROTO((FILE *t, register char *extra_headers));
 static int reply_to __APROTO((FILE *t, char *address));
-static void aux_param_bool __APROTO((FILE *f, char *name, int on));
-static void aux_param_str __APROTO((FILE *f, char *name, char *str));
-static void aux_param_int __APROTO((FILE *f, char *name, int i));
-static aux_sh __APROTO((article_header *ah, char *script, char *prog, char *action, char *record, char *sent_fmt, int append_sig));
 static void append_file __APROTO((char *name, register FILE *f));
 static char *get_distr __APROTO((char *orig, char *distr));
 static char *inet_name __APROTO((void));
@@ -51,13 +43,8 @@ export char *mail_record	= NULL;
 export char *news_record	= NULL;
 export char *mail_script	= NULL;
 export char *news_script	= NULL;
-export char *mailer_program	= REC_MAIL;
-export int  mailer_pipe_input	= 1;
 export char *inews_program	= NULL;
 export int  inews_pipe_input	= 1;
-export char *editor_program	= NULL;
-export char *spell_checker	= NULL;
-export char *mail_alias_expander = NULL;
 
 export char *bug_address	= BUG_REPORT_ADDRESS;
 
@@ -66,17 +53,13 @@ export int include_full_header	= 0;
 export int orig_to_include_mask	= 0x3;
 export int include_mark_blanks	= 0;
 
-export int empty_answer_check	= 1; /* reject replies that are not edited */
-export int response_check_pause = 0; /* time to wait for background cmds */
-export char *response_dflt_answer = "send";
 
 #ifdef APPEND_SIGNATURE
 export int append_sig_mail = 1; /* mail does not append it */
 #else
 export int append_sig_mail = 0;
 #endif
-export int append_sig_post = 0;	/* inews should always do this */
-export int query_signature = 1;	/* query user */
+export int append_sig_post = 1;	/* we should do this */
 
 
 #define INCL_MARK_SIZE	10
@@ -91,53 +74,16 @@ extern key_type help_key;
 
 extern regexp *pg_regexp;
 extern int pg_new_regexp;
-import int use_mmdf_folders;
-import int novice;
-import char *pager;
-import int data_bits;
 
-static int ed_line;
+int ed_line;
 
 #ifdef NNTP
-#ifdef NNTP_MINI_INEWS_HEADER
 #include <time.h>
+#include <pwd.h>
 
 extern struct tm *gmtime();
 extern char *asctime();
 
-static mini_inews_header(t)
-FILE *t;
-{
-    time_t now;
-    char *date, host[64];
-
-    now = cur_time();
-    date = asctime(gmtime(&now));
-    date[3] = date[7] = date[10] = date[19] = date[24] = NUL;
-#ifdef NNTP_PATH_HOSTNAME
-    strncpy(host, NNTP_PATH_HOSTNAME, 64);
-
-    if (host[0] == '/') {
-	FILE *fph;
-	if ((fph = open_file(host, OPEN_READ)) && fgets(host, 64, fph)) {
-	    char *cp;
-	    if (cp = strchr(host, '\n')) *cp = NUL;
-	} else {
-	    msg("Can't get outgoing hostname from file, using nn_gethostname");
-	    nn_gethostname(host, 64);
-	}
-	if (fph) fclose(fph);
-    }	
-#else
-    nn_gethostname(host, 64);
-#endif
-
-    fprintf(t, "Path: %s!%s\n", host, user_name());
-    fprintf(t, "Date: %s %s %s %s GMT\n", date+8, date+4, date+22, date+11);
-    fprintf(t, "Message-ID: <%s.%ld@%s>\n", user_name(), (long)now, host);
-    ed_line += 3;
-}
-#endif
 #endif
 
 static int
@@ -331,146 +277,6 @@ int get_regexp;
     return 1;
 }
 
-/*
- * invoke aux shell script with suitable arguments
- *
- * WARNING: record may be NULL, so it must be the last argument!!
- */
-
-static void
-aux_param_bool(f, name, on)
-FILE *f;
-char *name;
-int on;
-{
-    fprintf(f, "%s=%s\n", name, on ? "true" : "false");
-}
-
-static void
-aux_param_str(f, name, str)
-FILE *f;
-char *name, *str;
-{
-    int xport = (*name == '*');
-    
-    if (xport) name++;
-    fprintf(f, "%s='%s'\n", name, str != NULL ? str : "");
-    if (xport) fprintf(f, "export %s\n", name);
-}
-
-static void
-aux_param_int(f, name, i)
-FILE *f;
-char *name;
-int i;
-{
-    fprintf(f, "%s=%d\n", name, i);
-}
-
-static int
-aux_sh(ah, script, prog, action, record, sent_fmt, append_sig)
-article_header *ah;
-char *script, *prog, *action, *record, *sent_fmt;
-int append_sig;
-{
-    FILE *param;
-    char *args[10], *fn;
-    char route[512], *poster;
-    register char **ap = args;
-
-    if (strcmp(prog, "COMPLETE") == 0) goto no_params;
-
-    param = open_file(relative(nn_directory, ".param"), OPEN_CREATE);
-    if (param == NULL) {
-	strcpy(delayed_msg, "cannot create .param file for aux script");
-	return 1;
-    }
-
-    if (getenv("LOGNAME") == NULL)
-	aux_param_str(param, "*LOGNAME", user_name());
-
-    if (strcmp(prog, "cancel") == 0) {
-	aux_param_str(param, "ART_ID", action);	/* article id for cancel */
-	aux_param_str(param, "GROUP", record);	/* group name for cancel */
-	if (news.ng_dist)
-	  aux_param_str(param, "DIST", news.ng_dist); /* dist for cancel */
-	else
-	  aux_param_str(param, "DIST", "none"); /* dist for cancel */
-    } else {
-	aux_param_str(param, "FIRST_ACTION", action);
-	aux_param_str(param, "RECORD", record);
-	aux_param_str(param, "WORK", temp_file);
-	aux_param_int(param, "ED_LINE", ed_line);
-
-	aux_param_bool(param, "NOVICE", novice);
-	aux_param_bool(param, "EMPTY_CHECK", empty_answer_check);
-	aux_param_bool(param, "APPEND_SIG", append_sig);
-	aux_param_bool(param, "QUERY_SIG", append_sig && query_signature);
-
-	if (editor_program != NULL)
-	    aux_param_str(param, "EDITOR", editor_program);
-	aux_param_str(param, "SPELL_CHECKER", spell_checker);
-	aux_param_str(param, "ALIAS_EXPANDER", mail_alias_expander);
-	aux_param_str(param, "PAGER", pager);
-	aux_param_str(param, "MAILER", mailer_program);
-	aux_param_bool(param, "MAILER_PIPE", mailer_pipe_input);
-	aux_param_int(param, "WAIT_PERIOD", response_check_pause);
-	aux_param_str(param, "DFLT_ANSW", response_dflt_answer);
-	aux_param_str(param, "POST", inews_program);
-	aux_param_bool(param, "POST_PIPE", inews_pipe_input);
-	aux_param_str(param, "MMDF_SEP", use_mmdf_folders ? "\1\1\1\1" : "");
-
-	aux_param_str(param, "CHSET_NAME", curchset->cs_name);
-	if (curchset->cs_width != 0)
-	    aux_param_int(param, "CHSET_WIDTH", curchset->cs_width);
-	else
-	    aux_param_int(param, "CHSET_WIDTH", data_bits);
-
-	if (current_group != NULL) {
-	    aux_param_str(param, "*G", current_group->group_name);
-	    if (ah == NULL)
-		fn = NULL;
-	    else
-#ifdef NNTP
-	    if (use_nntp)
-		fn = nntp_get_filename(ah->a_number, current_group);
-	    else
-#endif
-		fn = group_path_name;
-	    aux_param_str(param, "*A", fn != NULL ? fn : "");
-	}
-
-	poster = NULL;
-	if (ah != NULL && strcmp(prog, "follow") == 0) {
-	    poster = news.ng_reply != NULL ? news.ng_reply : news.ng_from;
-	    if (poster && reroute(route, poster)) 
-		poster = route;
-	    else
-		poster = NULL;
-	}
-	aux_param_str(param, "POSTER_ADR", poster);
-    }
-
-    fclose(param);
-
- no_params:
-    stop_usage();
-
-    /* OBS: relative() returns ptr to static data below */
-    *ap++ = "nnaux";
-    *ap++ = script != NULL ? script : relative(lib_directory, "aux");
-    *ap++ = prog;
-    *ap++ = temp_file;
-    *ap++ = NULL;
-
-    if (execute(SHELL, args, 1)) {
-	sprintf(delayed_msg, sent_fmt, " not");
-	return 1;
-    }
-    sprintf(delayed_msg, sent_fmt, "");
-    return 0;
-}
-
 static void
 append_file(name, f)
 char *name;
@@ -656,11 +462,6 @@ int incl;	/* <0: ask, 0: don't include, >0: include article */
 
 	nn_st_flag = A_ST_FOLLOW;
 
-#ifdef NNTP
-#ifdef NNTP_MINI_INEWS_HEADER
-	mini_inews_header(t);
-#endif
-#endif
 	ng_line(t, 1);
 
 	if (news.ng_subj)
@@ -796,7 +597,8 @@ int incl;	/* <0: ask, 0: don't include, >0: include article */
     if (aux_sh(ah, script, pgm, first_action, record_file,
 	       command == K_FOLLOW_UP ? "Article%s posted" : "Mail%s sent",
 	       append_sig) == 0)
-	if (ah) ah->flag |= nn_st_flag;
+	if (ah)
+	    ah->flag |= nn_st_flag;
 
     return edit_message;
 
@@ -1074,11 +876,6 @@ post_menu()
 	prompt("\1WAIT\1");
 
     ed_line = 3;
-#ifdef NNTP
-#ifdef NNTP_MINI_INEWS_HEADER
-    mini_inews_header(t);
-#endif
-#endif
     fprintf(t, "Newsgroups: %s\n", group_name);
     if (strcmp(distribution, "world")) {
 	fprintf(t, "Distribution: %s\n", distribution);
