@@ -55,22 +55,6 @@
 
 #define HAVE_STRCHR			/* */
 
-/*
- *	Availability of varargs/vprintf/vsprintf is normally assumed
- *	on most systems, *unless* NO_VARARGS is defined either here or
- *	in the m- file.
- *
- *	To overrule a NO_VARARGS in the m- file, define HAVE_VARARGS
- *	below if the varargs feature is available, *including* the
- *	vprintf/vsprintf routines.
- *
- *	You may also define NO_VARARGS here, if varargs or the v*printf
- *	are not available on this OS, but the m- file doesn't say so.
- */
-
-/* #define HAVE_VARARGS /* Ignore NO_VARARGS -- else default */
-/* #define NO_VARARGS /* */
-
 /*
  *	Define if a signal handler has type void (see signal.h)
  */
