@@ -4,31 +4,33 @@
  *	Global declarations and auxiliary functions.
  */
 
+#include <stdarg.h>
 #include <signal.h>
 #include <pwd.h>
+#include <sys/mman.h>
 #include "config.h"
 #include "patchlevel.h"
+#include "nn_term.h"
 
 /* global.c */
 
 static sig_type catch_keyboard __APROTO((int n));
 static sig_type catch_pipe __APROTO((int n));
 static sig_type catch_suspend __APROTO((int n));
-static enter_log __APROTO((int type, va_list zZap));
 static void mail_sys_error __APROTO((char *err, int isfatal));
 static void mem_error __APROTO((int t, int32 bytes));
 
 export char *home_directory;
-export char *nn_directory;		/* ~/.nn */
+export char *nn_directory;		/* ~/.nn as default */
 export char *news_directory;		/* /usr/spool/news */
-export char *news_lib_directory; 	/* /usr/lib/news */
+export char *news_lib_directory;	/* /usr/lib/news */
 export char *lib_directory;		/* /usr/local/lib/nn */
 export char *master_directory;		/* = lib */
 export char *help_directory;		/* = lib/help */
 export char *bin_directory = BIN_DIRECTORY;
 
 export char *db_directory;	/* /usr/spool/nn or NEWS_DIR/.nn */
-export char *db_data_directory;	/* ..../DATA     or undefined    */
+export char *db_data_directory;	/* ..../DATA	 or undefined	 */
 export int db_data_subdirs = 0;	/* set if DATA/[0-9]/ exist	 */
 
 export char *news_active;	/* NLIB/active or DB/ACTIVE */
@@ -41,7 +43,7 @@ export char *log_entry_filter = NULL;
 export char *temp_file;
 
 #ifndef TMP_DIRECTORY
-#define TMP_DIRECTORY "/usr/tmp"
+#define TMP_DIRECTORY "/var/tmp"
 #endif
 export char *tmp_directory = TMP_DIRECTORY;
 
@@ -51,8 +53,8 @@ export char version_id[100];
 export int use_nntp = 0;	/* bool: t iff we use nntp */
 #endif
 
-export unsigned short user_eid;
-export unsigned short user_id, group_id;
+uid_t	user_id;
+gid_t	group_id;
 export int process_id;
 export int who_am_i;
 export int dont_write_console = 0;
@@ -67,14 +69,12 @@ extern char *getlogin(), *getenv();
 #ifndef NGROUPS
 #include <sys/param.h>
 #endif
-#ifndef GIDSET_TYPE
-#define GIDSET_TYPE int
-#endif
-static int ngroups;
-static GIDSET_TYPE gidset[NGROUPS];
 
-static in_grplist(gid)
-GIDSET_TYPE gid;
+static uid_t	user_eid;
+static int	ngroups;
+static gid_t	gidset[NGROUPS];
+
+static int in_grplist(gid_t gid)
 {
     int n;
 
@@ -117,7 +117,7 @@ int n;
 
 #ifdef FAKE_INTERRUPT
     if (fake_keyb_siglist[n] && arm_fake_keyb_sig)
-        longjmp(fake_keyb_sig, 1);
+	longjmp(fake_keyb_sig, 1);
 #endif
 }
 
@@ -144,7 +144,7 @@ int n;
 #endif
 #ifdef FAKE_INTERRUPT
     if (fake_keyb_siglist[n] && arm_fake_keyb_sig)
-        longjmp(fake_keyb_sig, 1);
+	longjmp(fake_keyb_sig, 1);
 #endif
 }
 
@@ -190,7 +190,7 @@ init_global()
 #endif
 
     if (who_am_i != I_AM_NN) {
-	signal(SIGINT,  SIG_IGN);
+	signal(SIGINT,	SIG_IGN);
 	signal(SIGQUIT, SIG_IGN);
     }
     signal(SIGTERM, catch_hangup);
@@ -299,15 +299,20 @@ init_global()
     if ((pager = getenv("PAGER")) == NULL)
 	pager = DEFAULT_PAGER;
 
-    nn_directory = mk_file_name(home_directory, ".nn");
+    if(!nn_directory) /* if not set on command line */
+	nn_directory = mk_file_name(home_directory, ".nn");
 
+    return 0;
+}
+
+int init_global2()
+{
     if (who_am_i != I_AM_ADMIN && !file_exist(nn_directory, "drwx")) {
 	if (who_am_i != I_AM_NN) return -1;
 	if (mkdir(nn_directory, 0755) < 0)
 	    nn_exitmsg(1, "Cannot create %s directory", nn_directory);
 	return 1;
     }
-
     return 0;
 }
 
@@ -348,7 +353,7 @@ int mode;
 
      case OPEN_UPDATE:
 
-/*	f = fopen(name, "r+"); 	-- not reliable on many systems (sigh) */
+/*	f = fopen(name, "r+");	-- not reliable on many systems (sigh) */
 
 	if ((fd = open(name, O_WRONLY)) >= 0) {
 	    if ((f = fdopen(fd, "w")) != NULL) return f;
@@ -443,7 +448,7 @@ register FILE *f;
 }
 	
 /*
- * 	relative -- concat directory name and file name
+ *	relative -- concat directory name and file name
  */
 
 char *relative(dir, name)
@@ -475,7 +480,8 @@ char *dir;
 	if (*dir == '/')
 	    return copy_str(dir);
 	else {
-	    if (*dir == '~' && *++dir == '/') dir++;
+	    if (*dir == '~' && *++dir == '/')
+		dir++;
 	    return mk_file_name(home_directory, dir);
 	}
     }
@@ -563,6 +569,64 @@ char *mode;
     return statb.st_mtime;
 }
 
+/*
+ *	cmp_file: compare two files
+ *
+ *	Returns		0: The files are identical.
+ *			1: The files are different.
+ *			2: An error occurred.
+ */
+
+int cmp_file(work, copy)
+char *work, *copy;
+{
+    int fd1, fd2;
+    struct stat sb1, sb2;
+    u_char *p1, *p2;
+    off_t len1, len2;
+
+    if ((fd1 = open(work, O_RDONLY, 0)) < 0) {
+	msg("%s %s", work, strerror(errno));
+	return(2);
+    }
+    if ((fd2 = open(copy, O_RDONLY, 0)) < 0) {
+	msg("%s %s", copy, strerror(errno));
+	return(2);
+    }
+
+    if (fstat(fd1, &sb1)) {
+	msg("%s %s", work, strerror(errno));
+	return(2);
+    }
+    if (fstat(fd2, &sb2)) {
+	msg("%s %s", copy, strerror(errno));
+	return(2);
+    }
+
+    len1 = sb1.st_size;
+    len2 = sb2.st_size;
+
+    if (len1 != len2)
+	return(1);
+
+    if ((p1 = (u_char *)mmap(NULL, (size_t)len1, PROT_READ, MAP_SHARED, fd1, 0)) == (u_char *)-1) {
+	msg("work=%s %s", work, strerror(errno));
+	sleep(5);
+	return(2);
+    }
+    if ((p2 = (u_char *)mmap(NULL, (size_t)len2, PROT_READ, MAP_SHARED, fd2, 0)) == (u_char *)-1) {
+	msg("copy=%s %s", copy, strerror(errno));
+	sleep(5);
+	return(2);
+    }
+
+    for (; len1--; ++p1, ++p2) {
+	if (*p1 != *p2)
+	    return(1);
+    }
+    return(0);
+}
+
 /*
  * copy_file: copy (or append) src file to dest file.
  *
@@ -687,24 +751,18 @@ char *name, *suffix;
 
 
 static int
-enter_log(type, va_tail)
-char type;
-va_tdcl
+enter_log(int type, char *fmt, va_list ap)
 {
     FILE *log;
-    char *msg1, buf[512];
+    char buf[512];
 
-    if (log_entry_filter != NULL)
-	for (msg1 = log_entry_filter; *msg1; msg1++)
-	    if (*msg1 == type) return 1;
-
-    msg1  = va_arg1(char *);
-    vsprintf(buf, msg1, va_args2toN);
+    vsprintf(buf, fmt, ap);
 
     /* cannot use relative: one of the args may be generated by it */
 
     log = open_file(log_file, OPEN_APPEND);
-    if (log == NULL) return 0;
+    if (log == NULL)
+	return 0;
 
     fprintf(log, "%c: %s (%s): %s\n", type,
 	    date_time((time_t)0), user_name(), buf);
@@ -747,34 +805,32 @@ int isfatal;
     pclose(f);
 }
 
-/*VARARGS*/
-void sys_error(va_alist)
-va_dcl
+void
+sys_error(char *fmt,...)
 {
+    va_list	ap;
+    int		rval;
     char buf[512];
-    char *fmt;
 #ifndef HAVE_SYSLOG
     FILE *f;
 #endif
-    use_vararg;
 
-    start_vararg;
-    enter_log('E', va_args1toN);
-    end_vararg;
+    va_start(ap, fmt);
+    rval = enter_log('E', fmt, ap);
+    va_end(ap);
 
-    start_vararg;
-    fmt = va_arg1(char *);
-    vsprintf(buf, fmt, va_args2toN);
-    end_vararg;
+    va_start(ap, fmt);
+    vsprintf(buf, fmt, ap);
+    va_end(ap);
 
     if (who_am_i == I_AM_MASTER) {
 	mail_sys_error(buf, 1);
 	if (dont_write_console) nn_exit(7);
 #ifndef HAVE_SYSLOG
-	f = open_file("/dev/console", OPEN_CREATE);
-	if (f == NULL) nn_exit(8);
-	fprintf(f, "\n\rNNMASTER FATAL ERROR\n\r%s\n\n\r", buf);
-	fclose(f);
+        f = open_file("/dev/console", OPEN_CREATE);
+        if (f == NULL) nn_exit(8);
+        fprintf(f, "\n\rNNMASTER FATAL ERROR\n\r%s\n\n\r", buf);
+        fclose(f);
 #else /* HAVE_SYSLOG */
 	openlog("nnmaster", LOG_CONS, LOG_DAEMON);
 	syslog(LOG_ALERT, "%s", buf);
@@ -782,7 +838,10 @@ va_dcl
 #endif /* HAVE_SYSLOG */
 	nn_exit(7);
     }
-    nn_exitmsg(1, "%s", buf);
+    if (rval)
+	nn_exitmsg(1, "%s", buf);
+    else
+	nn_exitmsg(1, "Can't open log file!\n%s", buf);
 }
 
 /*
@@ -790,43 +849,45 @@ va_dcl
  *	instead of exit.  Clients still terminate!
  */
 
-/*VARARGS*/
 int
-sys_warning(va_alist)
-va_dcl
+sys_warning(char *fmt,...)
 {
+    va_list	ap;
+    int		rval;
     char buf[512];
-    char *fmt;
 #ifndef HAVE_SYSLOG
     FILE *f;
 #endif
     static char *last_err = NULL;
-    use_vararg;
 
-    start_vararg;
-    fmt = va_arg1(char *);
-    vsprintf(buf, fmt, va_args2toN);
-    end_vararg;
+    va_start(ap, fmt);
+    vsprintf(buf, fmt, ap);
+    va_end(ap);
 
     if (last_err != NULL) {
-	if (strcmp(last_err, buf) == 0) return -1;
+	if (strcmp(last_err, buf) == 0)
+	    return -1;
 	free(last_err);
     }
     last_err = copy_str(buf);
 
-    start_vararg;
-    enter_log('R', va_args1toN);
-    end_vararg;
+    va_start(ap, fmt);
+    rval = enter_log('R', fmt, ap);
+    va_end(ap);
 
-    if (who_am_i != I_AM_MASTER)
-	nn_exitmsg(1, "%s", buf);
+    if (who_am_i != I_AM_MASTER) {
+	if (rval)
+	    nn_exitmsg(1, "%s", buf);
+	else
+	    nn_exitmsg(1, "Can't open log file!\n%s", buf);
+    }
 
     mail_sys_error(buf, 0);
     if (dont_write_console) return -1;
 #ifndef HAVE_SYSLOG
     if((f = open_file("/dev/console", OPEN_CREATE)) != NULL) {
-	fprintf(f, "\n\rNNMASTER WARNING\n\r%s\n\n\r", buf);
-	fclose(f);
+        fprintf(f, "\n\rNNMASTER WARNING\n\r%s\n\n\r", buf);
+        fclose(f);
     }
 #else /* HAVE_SYSLOG */
     openlog("nnmaster", LOG_CONS, LOG_DAEMON);
@@ -836,18 +897,15 @@ va_dcl
     return -1;
 }
 
-/*VARARGS*/
 int
-log_entry(va_alist)
-va_dcl
+log_entry(int type, char *fmt,...)
 {
-    int type, rval;
-    use_vararg;
+    va_list	ap;
+    int		rval;
 
-    start_vararg;
-    type = va_arg1(int);
-    rval = enter_log(type, va_args2toN);
-    end_vararg;
+    va_start(ap, fmt);
+    rval = enter_log(type, fmt, ap);
+    va_end(ap);
 
     return rval;
 }
@@ -864,7 +922,7 @@ char *user_name()
 	user = getlogin();
 	if (user != NULL && *user != NUL) goto out;
 
-	pw = getpwuid((int)user_id);
+	pw = getpwuid(user_id);
 	if (pw != NULL && pw->pw_name[0] != NUL) {
 	    user = copy_str(pw->pw_name);
 	    goto out;
@@ -1060,7 +1118,7 @@ off_t len;
 #else
     if (stat(path, &st) < 0) return -1;
     fd = creat(path, st.st_mode & 07777);
-#endif    
+#endif	  
     if (fd < 0) return -1;
     close(fd);
     return 0;
@@ -1190,3 +1248,17 @@ FILE	*f;
 				/* possibly overwriting CR and/or NL */
   return buf;
 }
+
+/*
+ *	getline - gets a line from stdin
+ *			returns the length of the line
+ */
+
+int getline(char *line, int max)
+{
+    if (fgets(line, max, stdin) == NULL)
+	return 0;
+    else
+	return strlen(line);
+}
+
