@@ -25,7 +25,6 @@ static void update_last_new __APROTO((group_header *lastg));
 static article_number get_last_article __APROTO((group_header *gh));
 static void init_rctest __APROTO((group_header *gh, register struct rc_info *r));
 static attr_type rctest __APROTO((register article_header *ah, register struct rc_info *r));
-static void append __APROTO(());
 static void append_range __APROTO((int pp, int delim, article_number rmin, article_number rmax));
 static void begin_rc_update __APROTO((register group_header *gh));
 static void end_rc_update __APROTO((register group_header *gh));
@@ -107,7 +106,7 @@ static char *sel_path = NULL;
 
 /* line buffers */
 
-#define RC_LINE_MAX	8192
+#define RC_LINE_MAX	32768
 
 static char rcbuf[RC_LINE_MAX];
 static char selbuf[RC_LINE_MAX];
@@ -862,28 +861,23 @@ static article_number sel_max;	/* current select range max */
 static article_number sel_digest; /* current digest */
 static char sel_delim;		/* delimiter character */
 
-/*VARARGS*/
 static void
-append(va_alist)
-va_dcl
+append(int x, char *fmt, ...)
 {
-    int x;
+    va_list	ap;
     register char *p;
-    char **pp, *fmt;
-    use_vararg;
+    char **pp;
 
-    start_vararg;
-    x = va_arg1(int);
+    va_start(ap, fmt);
     pp = x ? &sel_p : &rc_p;
     p = *pp;
     if (p > (x ? &selbuf[RC_LINE_MAX - 16] : &rcbuf[RC_LINE_MAX - 16])) {
 	msg("%s line too long", x ? "select" : ".newsrc");
-	end_vararg;
+	va_end(ap);
 	return;
     }
-    fmt = va_arg2(char *);
-    vsprintf(p, fmt, va_args3toN);
-    end_vararg;
+    vsprintf(p, fmt, ap);
+    va_end(ap);
 
     while (*p) p++;
     *p = NL;
