@@ -25,7 +25,7 @@ static do_nnspew __APROTO((void));
 static void prt_version __APROTO((void));
 static void do_db_check __APROTO((void));
 
-
+int init_global2();
 
 import char *bin_directory;
 
@@ -44,7 +44,7 @@ import int
 export int
     article_limit = -1,
     also_read_articles = 0,
-    also_full_digest = 0,
+    also_full_digest = 1,
     batch_mode = 0,
     conf_auto_quit = 0,
     do_kill_handling = 1,
@@ -75,6 +75,7 @@ extern char *sbrk();
 #endif
 #ifdef NOV
 extern void novartdir();
+extern void novfilename();
 #endif
 
 static int
@@ -156,7 +157,7 @@ static int nn_locked()
     if (proto_lock(I_AM_NN, PL_SET)) {
 	
 	if (proto_host[0])
-	    nn_exitmsg(1, "\nnn is running on host %s\nor .nn/LOCK should be removed.\n\n", proto_host);
+	    nn_exitmsg(1, "\nnn is running on host %s\nor %s/LOCK should be removed.\n\n", nn_directory, proto_host);
 	else
 	    nn_exitmsg(1, "\nAnother nn process is already running\n\n");
     }
@@ -211,10 +212,10 @@ int upd;
 	return NULL;
     }
 
-    if (enter_last_read_mode == 0) goto none;
+    if (enter_last_read_mode == 0) return last;
 
     f = open_file(lg_file, OPEN_READ);
-    if (f == NULL) goto none;
+    if (f == NULL) return last;
     
     gh = NULL;
     if (fgets(buf, 256, f)) {
@@ -224,24 +225,24 @@ int upd;
     fclose(f);
 
     if (gh == NULL || gh == last || !(gh->group_flag & G_SEQUENCE))
-	goto none;
+	return last;
     
     switch (enter_last_read_mode) {
      case 1:		/* confirm if unread, skip if read */
-	if (gh->unread_count == 0) goto none;
+	if (gh->unread_count == 0) return last;
+	/* fall through */
      case 2:		/* confirm any case */
 	prompt_line = Lines-1;
 	prompt("Enter %s (%ld unread)? ", gh->group_name, gh->unread_count);
-	if (!yes(0)) goto none;
+	if (!yes(0)) return last;
 	break;
      case 3:		/* enter uncond if unread */
-	if (gh->unread_count == 0) goto none;
+	if (gh->unread_count == 0) return last;
+	/* fall through */
      case 4:		/* enter uncond */
 	break;
     }
     return gh;
- none:
-    return last;
 }
 
 static int
@@ -393,7 +394,7 @@ catch_up()
     fl;
     mode = 0;
 
-    if (gets(answer1)) {
+    if (fgets(answer1, sizeof(answer1), stdin)) {
 	if (strncmp(answer1, "auto", 4) == 0) {
 	    tprintf("\nUPDATING .newsrc FILE....");
 	    fl;
@@ -436,7 +437,7 @@ catch_up()
 	       (long)(gh->last_db_article - gh->last_article));
 	fl;
 
-	if (gets(answer1) == NULL || s_keyboard)
+	if (fgets(answer1, sizeof(answer1), stdin) == NULL || s_keyboard)
 	    *answer1 = 'q';
 
 	switch (*answer1) {
@@ -445,7 +446,7 @@ catch_up()
 	    tputc(NL);
 	    tprintf("Update rest? (yn) n\b");
 	    fl;
-	    if (gets(answer1) == NULL || *answer1 != 'y')
+	    if (fgets(answer1, sizeof(answer1), stdin) == NULL || *answer1 != 'y')
 		return;
 
 	    mode = -1;
@@ -648,7 +649,8 @@ do_nnspew()
 static void
 prt_version()
 {
-    tprintf("Release %s\n  by Kim F. Storm, Peter Wemm, and Michael T Pins, 1996\n\n", version_id);
+    clrdisp();
+    tprintf("Release %s\n  by Kim F. Storm, Peter Wemm, and Michael T Pins, 2002\n\n", version_id);
 }
 
 int
@@ -784,11 +786,15 @@ char *argv[];
 	who_am_i = I_AM_NN;
     }
 #ifdef NOV
-# ifdef NOV_DIRECTORY
-    if (who_am_i != I_AM_VIEW)
+    if (who_am_i != I_AM_VIEW) {
+#ifdef NOV_DIRECTORY
 	novartdir(NOV_DIRECTORY);
-# endif
-#endif
+#endif	/* NOV_DIRECTORY */
+#ifdef NOV_FILENAME
+	novfilename(NOV_FILENAME);
+#endif	/* NOV_FILENAME */
+    }
+#endif	/* NOV */
 
     if (who_am_i == I_AM_NN || (who_am_i == I_AM_ADMIN && argc == 1)) {
 	if (!batch_mode && !isatty(0)) {
@@ -834,8 +840,28 @@ char *argv[];
     }
 #endif
 
+	/* do this before nntp_check so nntp_server can be set on cmdline as
+	 * nn -someoptions nntp_server=[somehost OR /somefile] -moreoptionsmaybe
+	 *
+	 * You really want to do that if you are using multiple servers and/or
+	 * .newsrc files.  The downside is that you can't override an init
+	 * file setting on the command line, but this is something I very
+	 * rarely have seen a need for.
+	 *
+	 * Somewhat gross; we also look for the -N and -x options here,
+	 * since we don't want to fail in nn_locked() if these options
+	 * are specified, and I want to check for the lock before getting
+	 * the active file, and the documented behavior is to not process
+	 * command line options before the init file is read.  The issue would
+	 * go away if nn no longer retrieved the whole active file.  One of
+	 * these days...
+	*/
+
     if ((say_welcome = init_global()) < 0)
 	nn_exitmsg(1, "%s: nn has not been invoked to initialize user files", pname);
+    parseargv_variables(argv, argc);
+    if ((say_welcome = init_global2()) < 0)
+	nn_exitmsg(1, "%s: nn has not been invoked to initialize user files", pname);
 
 #ifdef NNTP
     nntp_check();
@@ -847,6 +873,8 @@ char *argv[];
 #endif
     init_macro();
 
+    if (nn_locked()) nn_exit(2);
+
  nnspew:
 
     if (who_am_i != I_AM_VIEW)
@@ -858,8 +886,8 @@ char *argv[];
 	/*FALLTHROUGH*/
 
      case I_AM_NN:
-	visit_init_file(0, argv[1]);
 	init_term(1);
+	visit_init_file(0, argv[1]);
 
 	if (say_welcome) {
 	    first_time_user = 1;
@@ -924,6 +952,7 @@ char *argv[];
 	}
 	admin_mode(argv[1]);
 	nn_exit(0);
+	break;
 
      case I_AM_CHECK:
 	init_term(0);
@@ -962,6 +991,7 @@ char *argv[];
      case I_AM_POST:
 	do_nnpost(argc, argv);
 	nn_exit(0);
+	break;
 
      case I_AM_SPEW:
 	if (proto_lock(I_AM_SPEW, PL_SET) == 0) {
@@ -970,6 +1000,7 @@ char *argv[];
 	} else
 	    cmd = 1;	/* don't interfere with other nnspew */
 	nn_exit(cmd);
+	break;
     }
 
     if (!silent && who_am_i != I_AM_CHECK)
@@ -983,7 +1014,11 @@ char *argv[];
 	nn_exit(0);
     }
 
-    if (nn_locked()) nn_exit(2);
+        /* at one point, the command line variable settings were
+         * handled in the next routine (named_group_sequence()), which
+         * meant you couldn't change the .newsrc file.  I moved that
+         * processing into parseargv_variables(), which is called quite
+         * early.  Olson, 11/99 */
 
     visit_rc_file();
 
