@@ -11,12 +11,14 @@
  * any mistakes are mine :-)  ++Kim
  */
 
-
+#include <stdarg.h>
+#include <sys/param.h>
 #include "config.h"
+#include "nn_term.h"
 
 /*
- * 	nn maintains a cache of recently used articles to improve efficiency.
- * 	To change the size of the cache, define NNTPCACHE in config.h to be
+ *	nn maintains a cache of recently used articles to improve efficiency.
+ *	To change the size of the cache, define NNTPCACHE in config.h to be
  *	the new size of this cache.
  */
 
@@ -27,11 +29,16 @@
 #ifdef NNTP
 #include <stdio.h>
 #include "nntp.h"
+#include "patchlevel.h"
 #include <sys/socket.h>
+#include <string.h>
+#include <strings.h>
 #ifndef EXCELAN
 #include <netdb.h>
 #endif
 #include <errno.h>
+#include <pwd.h>
+#include <ctype.h>
 
 #ifdef NOV
 #include "hash.h"
@@ -50,47 +57,55 @@
 #endif
 #endif
 
-
 /* nntp.c */
 
-static debug_msg __APROTO((char *prefix, char *str));
-static io_error __APROTO((void));
+static int	reconnect_server(int);
+static int	connect_server(void);
+static void debug_msg __APROTO((char *prefix, char *str));
 static void find_server __APROTO((void));
-static get_server_line __APROTO((char *string, int size));
-static get_server __APROTO((char *string, int size));
-static get_socket __APROTO((void));
-static connect_server __APROTO((void));
-static put_server __APROTO((char *string));
-static ask_server __APROTO(());
-static copy_text __APROTO((register FILE *fp));
-static do_set_group __APROTO((void));
+static int get_server_line __APROTO((char *string, int size));
+static int get_server __APROTO((char *string, int size));
+static int get_socket __APROTO((void));
+static void put_server __APROTO((char *string));
+static int copy_text __APROTO((register FILE *fp));
 static struct cache *search_cache __APROTO((article_number art, group_header *gh));
 static struct cache *new_cache_slot __APROTO((void));
 static void clean_cache __APROTO((void));
+static void nntp_doauth(void);
+#ifndef NOV
 static sort_art_list __APROTO((register article_number *f1, register article_number *f2));
+#endif	/* !NOV */
 
 
 import char *db_directory, *tmp_directory, *news_active;
 
-export char nntp_server[256];	/* name of nntp server */
+
+static char host_name[MAXHOSTNAMELEN];
+static char domain[MAXHOSTNAMELEN];
+static char last_put[NNTP_STRLEN];
+
 export int nntp_failed = 0;	/* bool: t iff connection is broken in
 				   nntp_get_article() or nntp_get_active() */
 
 export int nntp_cache_size = NNTPCACHE;
 export char *nntp_cache_dir = NULL;
+export char *nntp_server = NNTP_SERVER;	/* name of nntp server */
+export char *nntp_user, *nntp_password; /* so can set on command line */
 
 export int nntp_local_server = 0;
 export int nntp_debug = 0;
 
+import char *home_directory;
 import int silent, no_update;
 
-import int sys_nerr;
+#ifndef __FreeBSD__
 #ifndef __NetBSD__
-import char *sys_errlist[];
+#ifndef __linux__
+extern int sys_nerr;
+extern char *sys_errlist[];
+#endif	/* __linux__ */
 #endif	/* __NetBSD__ */
-extern void nn_exitmsg();
-extern void sys_error();
-extern int sys_warning();
+#endif	/* __FreeBSD__ */
 
 #define syserr() (errno >= 0 && errno < sys_nerr ? \
 		  sys_errlist[errno] : "Unknown error.")
@@ -99,18 +114,18 @@ import char *mktemp();
 
 static FILE *nntp_in = NULL;		/* fp for reading from server */
 static FILE *nntp_out = NULL;		/* fp for writing to server */
+static int reconnecting = 0;
 static int is_connected = 0;		/* bool: t iff we are connected */
+static int need_auth = 0;
 static group_header *group_hd;		/* ptr to servers current group */
-static int group_is_set = 0;		/* bool: t iff group_hd is set */
-static int try_again = 0;		/* bool: t if timeout forces retry */
 static int can_post = 0;		/* bool: t iff NNTP server accepts postings */
 
 #define ERR_TIMEOUT	503		/* Response code for timeout */
 					/* Same value as ERR_FAULT */
 
 #ifdef NO_RENAME
-static rename(old, new)
-char *old, *new;
+static int
+rename(char *old, char *new)
 {
     if (unlink(new) < 0 && errno != ENOENT) return -1;
     if (link(old, new) < 0) return -1;
@@ -126,12 +141,12 @@ char *old, *new;
  *
  *	This is controlled via the nntp-debug variable in nn, and
  *	the option -D2 (or -D3 if the normal -D option should also
- *	be turned on).  Debug output from the master is written in
+ *	be turned on).	Debug output from the master is written in
  *	$TMP/nnmaster.log.
  */
 
-static debug_msg(prefix, str)
-char *prefix, *str;
+static void
+debug_msg(char *prefix, char *str)
 {
     static FILE *f = NULL;
     
@@ -149,44 +164,21 @@ char *prefix, *str;
     }
 
     msg("NNTP%s %s", prefix, str);
-    user_delay(1);
+    user_delay(3);
 }
 
-/*
- * io_error: signal an I/O error in talking to the server.
- *
- * 	An nn client terminates a session with the user.  The master
- *	simply closes the connection.  The flag nntp_failed is set, for
- *	use by the master to terminate collection.
- *
- *	BUG: if the nntp server is forcibly killed, errno can contain a
- *	bogus value, resulting in strange error messages.  It is
- *	probably better just to write out the numerical value of errno.
- */
-
-static io_error()
-{
-    if (who_am_i != I_AM_MASTER) {
-	nn_exitmsg(1, "Lost connection to NNTP server %s: %s", nntp_server, syserr());
-        /* NOTREACHED */
-    }
-    nntp_failed = 1;
-    if (is_connected) {
-	log_entry('N', "Lost connection to server %s: %s", nntp_server, syserr());
-	nntp_close_server();
-    }
-}
 
 /*
  * find_server: Find out which host to use as NNTP server.
  *
- * 	This is done by consulting the file NNTP_SERVER (defined in
- * 	config.h).  Set nntp_server[] to the host's name.
+ *	This is done by consulting the file NNTP_SERVER (defined in
+ *	config.h).  Set nntp_server[] to the host's name.
  */
 
-static void find_server()
+static void
+find_server()
 {
-    char *cp, *name, *getenv();
+    char *cp, *name;
     char buf[BUFSIZ];
     FILE *fp;
 
@@ -202,14 +194,14 @@ static void find_server()
      * Let the user worry about keeping his .newsrc straight.
      */
     if ((cp = getenv("NNTPSERVER")) != NULL) {
-	strncpy(nntp_server, cp, sizeof nntp_server);
+	nntp_server = cp;
 	return;
     }
+    else if(*nntp_server != '/')
+	return; /* variable was set on cmd line, or in init file */
 #endif /* NOV */
 
-    name = NNTP_SERVER;
-    if (*name != '/')
-	name = relative(lib_directory, name);
+    name = nntp_server; /* default, or variable was set to a filename */
 
     if ((fp = open_file(name, OPEN_READ)) != NULL) {
 	while (fgets(buf, sizeof buf, fp) != 0) {
@@ -217,7 +209,9 @@ static void find_server()
 		continue;
 	    if ((cp = strchr(buf, '\n')) != 0)
 		*cp = '\0';
-	    strncpy(nntp_server, buf, sizeof nntp_server);
+	    nntp_server = strdup(buf);
+	    if(!nntp_server)
+		sys_error("Failed to allocate space for name of NNTP server!");
 	    fclose(fp);
 	    return;
 	}
@@ -233,21 +227,19 @@ static void find_server()
 /*
  * get_server_line: get a line from the server.
  *
- * 	Expects to be connected to the server.
- * 	The line can be any kind of line, i.e., either response or text.
+ *	Expects to be connected to the server.
+ *	The line can be any kind of line, i.e., either response or text.
  *	Returns length of line if no error.
  *	If error and master, then return -1, else terminate.
  */
 
 static int
-get_server_line(string, size)
-    register char *string;
-    register int size;
+get_server_line(register char *string, register int size)
 {
-    if (fgets(string, size, nntp_in) == NULL) {
-	io_error();
-	return -1;
-    }
+    int retry = 2;
+
+    while (fgets(string, size, nntp_in) == NULL)
+	    retry = (reconnect_server(retry));
 
     size = strlen(string);
     if (size < 2 || !(string[size-2] == '\r' && string[size-1] == '\n'))
@@ -257,16 +249,16 @@ get_server_line(string, size)
     return size-2;
 }
 
+
 /*
  * get_server: get a response line from the server.
  *
- * 	Expects to be connected to the server.
- * 	Returns the numerical value of the reponse, or -1 in case of errors.
+ *	Expects to be connected to the server.
+ *	Returns the numerical value of the reponse, or -1 in case of errors.
  */
 
-static get_server(string, size)
-    char *string;
-    int size;
+static int
+get_server(char *string, int size)
 {
     if (get_server_line(string, size) < 0)
 	return -1;
@@ -277,7 +269,7 @@ static get_server(string, size)
 }
 
 /*
- * get_socket:  get a connection to the nntp server.
+ * get_socket:	get a connection to the nntp server.
  *
  * Errors can happen when YP services or DNS are temporarily down or
  * hung, so we log errors and return failure rather than exitting if we
@@ -285,21 +277,22 @@ static get_server(string, size)
  * the -r interval is) are not that bad.  Dave Olson, SGI
  */
 
-static get_socket()
+static int
+get_socket()
 {
     int s;
     struct sockaddr_in sin;
 #ifndef EXCELAN
-    struct servent *getservbyname(), *sp;
-    struct hostent *gethostbyname(), *hp;
+    struct servent *sp;
+    struct hostent *hp;
 
 #ifdef h_addr
     int     x = 0;
     register char **cp;
 #endif /* h_addr */
 
-    if ((sp = getservbyname("nntp", "tcp")) ==  NULL)
-	return sys_warning("nntp/tcp: Unknown service.\n");
+	if ((sp = getservbyname("nntp", "tcp")) ==  NULL)
+	    return sys_warning("nntp/tcp: Unknown service.\n");
 
     s = who_am_i == I_AM_MASTER ? 10 : 2;
     while ((hp = gethostbyname(nntp_server)) == NULL) {
@@ -317,7 +310,7 @@ static get_socket()
     clearobj(&sin, sizeof(sin), 1);
     sin.sin_family = AF_INET;
     sin.sin_port = htons(0);
-#endif  /* EXCELAN */
+#endif	/* EXCELAN */
 
 #ifdef h_addr
     /* get a socket and initiate connection -- use multiple addresses */
@@ -371,7 +364,7 @@ static get_socket()
 #endif /* NO_MEMMOVE */
     if (connect(s, (struct sockaddr *) &sin, sizeof(sin)) < 0)
 	goto conn_err;
-#endif  /* EXCELAN */
+#endif	/* EXCELAN */
 #endif /* h_addr */
     return s;
 
@@ -390,21 +383,26 @@ static get_socket()
     return -1;
 }
 
+
 /*
  * connect_server: initialise a connection to the nntp server.
  *
- * 	It expects nntp_server[] to be set previously, by a call to
- * 	nntp_check.  It is called from nntp_get_article() and
+ *	It expects nntp_server[] to be set previously, by a call to
+ *	nntp_check.  It is called from nntp_get_article() and
  *	nntp_get_active() if there is no established connection.
+ *	This gets called at initialization, and whenever it looks
+ *	like the server may have closed the connection on us.
  */
 
-static connect_server()
+static int
+connect_server()
 {
     int sockt_rd, sockt_wr;
-    int response, response2;
+    int response;
+    int triedauth = 0;
     char line[NNTP_STRLEN];
-    
-    if (who_am_i != I_AM_MASTER && !silent)
+
+    if (who_am_i != I_AM_MASTER && !silent && !reconnecting)
 	msg("Connecting to NNTP server %s ...", nntp_server);
 
     nntp_failed = 1;
@@ -416,15 +414,14 @@ static connect_server()
 
     if ((nntp_in = fdopen(sockt_rd, "r")) == NULL) {
 	close(sockt_rd);
-        return -1;
+	return -1;
     }
-    setvbuf(nntp_in, NULL, _IOLBF, 0);
     sockt_wr = dup(sockt_rd);
     if ((nntp_out = fdopen(sockt_wr, "w")) == NULL) {
 	close(sockt_wr);
 	fclose(nntp_in);
-        nntp_in = NULL;               /* from above */
-        return -1;
+	nntp_in = NULL; 	      /* from above */
+	return -1;
     }
 
     /* Now get the server's signon message */
@@ -438,8 +435,13 @@ static connect_server()
 	    fclose(nntp_in);
 	    return -1;
 	}
+    } else if (reconnecting && need_auth) {
+	nntp_doauth();
+	triedauth++;
     } else {
 	switch (response) {
+	case OK_AUTH:
+	    break;
 	case OK_CANPOST:
 	    can_post = 1;
 	    break;
@@ -452,59 +454,118 @@ static connect_server()
 	}
     }
 
+retrymode:
     /* Try and speak NNRP style reader protocol */
     sprintf(line, "MODE READER");
-    if (put_server(line) >= 0) {
-        /* See what we got if from a NNRP compiant server like INN's nnrpd */
-        response = get_server(line, sizeof(line));
+    put_server(line);
+    /* See what we got if from a NNRP compiant server like INN's nnrpd */
+    response = get_server(line, sizeof(line));
 
-	switch (response) {
-	case OK_CANPOST:
-	    can_post = 1;
-	    break;
-	case OK_NOPOST:
-	    can_post = 0;
-	    break;
-	case ERR_COMMAND:
-	default:
-	    /* if it doesn't understand MODE READER, we dont care.. :-) */
-	    break;
+    switch (response) {
+    case OK_CANPOST:
+	can_post = 1;
+	break;
+    case OK_NOPOST:
+	can_post = 0;
+	break;
+    case ERR_NEEDAUTH:
+	if(!triedauth) {
+	    nntp_doauth();
+	    triedauth++;
+	    goto retrymode;
 	}
+	break;	/* will probably fail hard later */
+    case ERR_COMMAND:
+    default:
+	/* if it doesn't understand MODE READER, we dont care.. :-) */
+	break;
     }
 
-    if (who_am_i != I_AM_MASTER && !silent)
+    if(!can_post && !triedauth) {
+	/* sometimes servers say no posting until auth done, so try.
+	 * it might be better if this was controlled by some variable,
+	 * since some people are OK with readonly access */
+	msg("No post, so trying authorization; if readonly OK, just skip");
+	nntp_doauth();
+	triedauth++;
+	goto retrymode;
+    }
+
+    if (who_am_i != I_AM_MASTER && !silent && !reconnecting)
 	msg("Connecting to NNTP server %s ... ok (%s)",
 	    nntp_server, can_post ? "posting is allowed" : "no posting");
 
     is_connected = 1;
-    group_is_set = 0;
     nntp_failed = 0;
-    try_again = 0;
     return 0;
 }
 
 
 /*
- * put_server:  send a line to the nntp server.
+ * reconnect_server -- reconnect to server after a timeout.
+ * Re-establish correct group and resend last command to get back to the
+ * correct state.
  *
- * 	Expects to be connected to the server.
+ *	Returns:	-1 if the error is fatal (and we should exit).
+ *			# of retries left otherwise.
+ *
+ *	Side Effects:	If fails to connect and retry==0, calls nn_exitmsg.
  */
 
-static put_server(string)
-    char *string;
+static int
+reconnect_server(int retry)
 {
-    if (nntp_debug) debug_msg(">>>", string);
+    char buf[NNTP_STRLEN];
 
-    fprintf(nntp_out, "%s\r\n", string);
-    if (fflush(nntp_out) == EOF) {
-	io_error();
-	return -1;
+    retry--;
+    msg("Reconnecting");
+    if (nntp_debug) {
+	msg("retry = %d", retry);
+	user_delay(2);
     }
-    return 0;
+    reconnecting = 1;
+    strcpy (buf, last_put);
+    nntp_close_server();
+    if (connect_server() < 0) {
+	if (nntp_debug)
+	    debug_msg("failed to connect", "");
+	if (retry > 0)
+	    return(retry);
+	else
+	    nn_exitmsg(1, "failed to reconnect to server");
+    }
+
+    if (nntp_set_group(group_hd) < 1)
+	nn_exitmsg(1, "unable to set group");
+
+    if (nntp_debug)
+	debug_msg(" Resend last command", buf);
+    put_server(buf);
+    reconnecting = 0;
+    return(retry);
+}
+
+
+/*
+ * put_server:	send a line to the nntp server.
+ *
+ *	Expects to be connected to the server.
+ */
+
+static void
+put_server(char *string)
+{
+    if (nntp_debug)
+	debug_msg(">>>", string);
+
+    strcpy(last_put, string);
+    fprintf(nntp_out, "%s\r\n", string);
+    (void) fflush(nntp_out);
+    return;
 }
 
 /*
- * ask_server:  ask the server a question and return the answer.
+ * ask_server:	ask the server a question and return the answer.
  *
  *	Expects to be connected to the server.
  *	Returns the numerical value of the reponse, or -1 in case of
@@ -515,56 +576,66 @@ static put_server(string)
 /* LIST XXX returns fatal ERR_FAULT code if requested list does not exist */
 /* This is only fatal for LIST ACTIVE -- else change to ERR_NOGROUPS */
 static int fix_list_response = 0;
+static char ask_reply[NNTP_STRLEN];
 
-/*VARARGS*/
-static ask_server(va_alist)
-va_dcl
+static int
+ask_server(char *fmt, ...)
 {
-    char buf[NNTP_STRLEN];
-    char *fmt;
+    char ask[NNTP_STRLEN];
     int response;
     int fix_err;
-    use_vararg;
+    va_list	ap;
 
     fix_err = fix_list_response;
     fix_list_response = 0;
 
-    start_vararg;
-    fmt = va_arg1(char *);
-    vsprintf(buf, fmt, va_args2toN);
-    end_vararg;
+    va_start(ap, fmt);
+    vsprintf(ask, fmt, ap);
+    va_end(ap);
 
-    if (put_server(buf) < 0)
-	return -1;
-    response = get_server(buf, sizeof(buf));
+    put_server(ask);
+    response = get_server(ask_reply, sizeof(ask_reply));
 
     /*
      * Handle the response from the server.  Responses are handled as
      * followes:
      *
-     * 100-199	Informational.  Passed back. (should they be ignored?).
+     * 100-199	Informational.	Passed back. (should they be ignored?).
      * 200-299	Ok messages.  Passed back.
      * 300-399	Ok and proceed.  Can not happen in nn.
      * 400-499	Errors (no article, etc).  Passed up and handled there.
+     * 480 (NEED_AUTH) handle specially.  Done here because some servers
+     * can send it at any point, after an idle period, as well as at
+     * various points during connect.
      * 500-599	Fatal NNTP errors.  Handled below.
      */
+
+    if (response == ERR_TIMEOUT) {
+	(void)reconnect_server(1);
+	response = get_server(ask_reply, sizeof(ask_reply));
+    }
+
+    if(response == ERR_NEEDAUTH) {
+	/* try auth, but just once, at least for now, and then retry
+	 * the original command, and continue */
+	nntp_doauth();
+	put_server(ask);
+	response = get_server(ask_reply, sizeof(ask_reply));
+    }
     if (response == ERR_GOODBYE || response > ERR_COMMAND) {
-	if (fix_err && response == ERR_FAULT) return ERR_NOGROUP;
+	if (fix_err && (response == ERR_FAULT || response == ERR_CMDSYN)) return ERR_NOGROUP;
 
 	nntp_failed = 1;
 	nntp_close_server();
 
-	if (response != ERR_TIMEOUT && response != ERR_GOODBYE) {
-			/* if not timeout and not goodbye, complain */
-	    sys_error("NNTP %s response: %d", buf, response);
+	if (response != ERR_GOODBYE) {
+			/* if not goodbye, complain */
+	    sys_error("NNTP %s response: %d", ask_reply, response);
 	    /* NOTREACHED */
 	}
 	if (response == ERR_GOODBYE) {
-	    sys_warning("NNTP %s response: %d", buf, response);
+	    sys_warning("NNTP %s response: %d", ask_reply, response);
 	}
-
-	try_again = 1;
-	group_is_set = 0;
     }
     return response;
 }
@@ -572,17 +643,17 @@ va_dcl
 /*
  * copy_text: copy text response into file.
  *
- * 	Copies a text response into an open file.
+ *	Copies a text response into an open file.
  *	Return -1 on error, 0 otherwise.  It is treated as an error, if
  *	the returned response it not what was expected.
  */
 
 static int last_copy_blank;
 
-static copy_text(fp)
-register FILE *fp;
+static int
+copy_text(register FILE *fp)
 {
-    char buf[NNTP_STRLEN];
+    char buf[NNTP_STRLEN*2];
     register char *cp;
     register int nlines;
 
@@ -610,30 +681,6 @@ register FILE *fp;
 }
 
 
-static do_set_group()
-{
-    int n;
-
-    switch (n = ask_server("GROUP %s", group_hd->group_name)) {
-     case OK_GROUP:
-	group_is_set = 1;
-	return 1;
-
-     case ERR_NOGROUP:
-	log_entry('N', "NNTP: group %s not found", group_hd->group_name);
-	return -1;
-
-     default:
-	if (try_again) return 0;	/* Handle nntp server timeouts */
-	break;
-    }
-    if (!nntp_failed) {
-	log_entry('N', "GROUP %s response: %d", group_hd->group_name, n);
-	nntp_failed = 1;
-    }
-    return -1;
-}
-
 /*
  * The following functions implements a simple lru cache of recently
  * accessed articles.  It is a simple way to improve effeciency.  Files
@@ -653,12 +700,11 @@ static unsigned time_counter = 1;		/* virtual time */
 /*
  * search_cache: search the cache for an (article, group) pair.
  *
- * 	Returns a pointer to the slot where it is, null otherwise
+ *	Returns a pointer to the slot where it is, null otherwise
  */
 
-static struct cache *search_cache(art, gh)
-    article_number art;
-    group_header *gh;
+static struct cache *
+search_cache(article_number art, group_header *gh)
 {
     struct cache *cptr = cache;
     int i;
@@ -678,16 +724,17 @@ static struct cache *search_cache(art, gh)
 /*
  * new_cache_slot: get a free cache slot.
  *
- * 	Returns a pointer to the allocated slot.
- * 	Frees the old filename, and allocates a new, unused filename.
+ *	Returns a pointer to the allocated slot.
+ *	Frees the old filename, and allocates a new, unused filename.
  *	Cache files can also stored in a common directory defined in
  *	~/.nn or CACHE_DIRECTORY if defined in config.h.
  */
 
-static struct cache *new_cache_slot()
+static struct cache *
+new_cache_slot()
 {
     register struct cache *cptr = cache;
-    int i, lru;
+    int i, lru = 0;
     unsigned min_time = time_counter;
     char name[FILENAME];
 
@@ -729,10 +776,11 @@ static struct cache *new_cache_slot()
 /*
  * clean_cache: clean up the cache.
  *
- * 	Removes all allocated files.
+ *	Removes all allocated files.
  */
 
-static void clean_cache()
+static void
+clean_cache()
 {
     struct cache *cptr = cache;
     int i;
@@ -745,24 +793,57 @@ static void clean_cache()
 /*
  * nntp_check: Find out whether we need to use NNTP.
  *
- * 	This is done by comparing the NNTP servers name with whatever
- * 	nn_gethostname() returns.
+ *	This is done by comparing the NNTP servers name with whatever
+ *	nn_gethostname() returns.
  *	use_nntp and news_active are initialised as a side effect.
  */
 
+void
 nntp_check()
 {
-    char host[128];
-    char *server_real_name;
+    char *server_real_name = NULL;
+    struct hostent *hp;
+#ifndef DOMAIN
+    char *cp;
+#endif
+
+    nn_gethostname(host_name, sizeof host_name);
+
+#ifndef DOMAIN
+    cp = index(host_name, '.');
+    if (cp)
+#ifdef HIDDENNET
+	strncpy(domain, ++cp, MAXHOSTNAMELEN);
+#else
+	strncpy(domain, host_name, MAXHOSTNAMELEN);
+#endif
+    else
+	nn_exitmsg(1, "hostname=%s, You need a fully qualified domain name",host_name);
+#else	/* DOMAIN */
+#ifdef HIDDENNET
+    strncpy(domain, DOMAIN, MAXHOSTNAMELEN);
+#else
+    strcpy(domain, host_name);
+    cp = index(domain, '.');
+    if (cp == NULL) {
+	strcat(domain, ".");
+	strncat(domain, DOMAIN, MAXHOSTNAMELEN - sizeof(host_name) - 1 );
+    }
+#endif	/* HIDDENNET */
+#endif	/* DOMAIN */
+
+    last_put[0] = '\0';
 
     if (nntp_local_server) return;
 
     find_server();
-    nn_gethostname(host, sizeof host);
-    strncpy(host, (gethostbyname(host))->h_name, sizeof host);
+    strncpy(host_name, (gethostbyname(host_name))->h_name, sizeof host_name);
 
-    server_real_name = (gethostbyname(nntp_server))->h_name;
-    use_nntp = (strcmp(host, server_real_name) != 0);
+    if ((hp = gethostbyname(nntp_server)) != NULL)
+	server_real_name = hp->h_name;
+    else
+	nn_exitmsg(1, "NNTPSERVER is invalid");
+    use_nntp = (strcmp(host_name, server_real_name) != 0);
 
     if (use_nntp) {
 	freeobj(news_active);
@@ -778,6 +859,7 @@ nntp_check()
  * nntp_no_post: Check to see whether posting is allowed.
  */
 
+int
 nntp_no_post()
 {
     if (!is_connected && connect_server() < 0)
@@ -791,26 +873,43 @@ nntp_no_post()
 
 
 /*
- * nntp_set_group: set the server's current group.
+ * nntp_set_group -- set the server's current group.
  *
- * 	Actual communication is delayed until an article is accessed, to
- * 	avoid unnecessary traffic.
+ *      Parameters:     gh is the structure containing the group name.
+ *
+ *      Returns:         1 if OK.
+ *                      -1 if no such group.
+ *                       0 otherwise.
  */
 
-nntp_set_group(gh)
-    group_header *gh;
+int
+nntp_set_group(group_header * gh)
 {
-    group_hd = gh;
-    group_is_set = 0;
-    return 0;
+    int             n;
+
+    switch (n = ask_server("GROUP %s", gh->group_name)) {
+        case OK_GROUP:
+            group_hd = gh;
+            return 1;
+
+        case ERR_NOGROUP:
+            log_entry('N', "NNTP: group %s not found\n", gh->group_name);
+            return -1;
+
+        default:        /* ??? returns this if connection down */
+            log_entry('N', "GROUP %s response: %d", group_hd->group_name, n);
+	    nntp_failed = 1;
+            return -1;           /* What did it return? */
+    }
 }
 
+
 #ifndef NOV
 /*
  * nntp_get_active:  get a copy of the active file.
  *
- * 	If we are the master get a copy of the file from the nntp server.
- * 	nnadmin just uses the one we already got.  In this way the master
+ *	If we are the master get a copy of the file from the nntp server.
+ *	nnadmin just uses the one we already got.  In this way the master
  *	can maintain a remote copy of the servers active file.
  *	We try to be a little smart, if not inefficient, about the
  *	modification times on the local active file.
@@ -818,6 +917,7 @@ nntp_set_group(gh)
  *	copy of the active file will be made for access via NFS.
  */
 
+int
 nntp_get_active()
 {
     FILE *old, *new;
@@ -828,7 +928,6 @@ nntp_get_active()
     if (who_am_i != I_AM_MASTER)
 	return access(news_active, 4);
 
- again:
     if (!is_connected && connect_server() < 0)
 	return -1;
 
@@ -848,7 +947,6 @@ nntp_get_active()
 	}
 	return -1;
      default:
-	if (try_again) goto again; /* Handle nntp server timeouts */
 	log_entry('N', "LIST response: %d", n);
 	return -1;
     }
@@ -883,7 +981,8 @@ nntp_get_active()
  *	Based on code from: olson%anchor.esd@sgi.com (Dave Olson)
  */
 
-FILE *nntp_get_newsgroups()
+FILE *
+nntp_get_newsgroups()
 {
     char *new_name;
     FILE *new;
@@ -893,7 +992,6 @@ FILE *nntp_get_newsgroups()
     new = open_file(new_name, OPEN_CREATE_RW|OPEN_UNLINK);
     if (new == NULL) return NULL;
 
- again:
     if (!is_connected && connect_server() < 0) goto err;
 
     fix_list_response = 1;
@@ -913,7 +1011,6 @@ FILE *nntp_get_newsgroups()
 	return NULL;
 
      default:
-	if (try_again) goto again; /* Handle nntp server timeouts */
 	log_entry('N', "LIST NEWSGROUPS response: %d", n);
 	goto err;
     }
@@ -929,7 +1026,7 @@ FILE *nntp_get_newsgroups()
 /*
  * nntp_get_article_list: get list of all article numbers in group
  *
- * 	Sends XHDR command to the server, and parses the following
+ *	Sends XHDR command to the server, and parses the following
  *	text response to get a list of article numbers which is saved
  *	in a list and returned.
  *	Return NULL on error.  It is treated as an error, if
@@ -939,14 +1036,14 @@ FILE *nntp_get_newsgroups()
 static article_number *article_list = NULL;
 static long art_list_length = 0;
 
-static sort_art_list(f1, f2)
-register article_number *f1, *f2;
+static int
+sort_art_list(register article_number *f1, register article_number *f2)
 {
     return (*f1 < *f2) ? -1 : (*f1 == *f2) ? 0 : 1;
 }
 
-article_number *nntp_get_article_list(gh)
-group_header *gh;
+article_number *
+nntp_get_article_list(group_header *gh)
 {
     char buf[NNTP_STRLEN];
     register article_number *art;
@@ -955,7 +1052,6 @@ group_header *gh;
     int n;
     static int try_listgroup = 1;
 
- again:
     if (!is_connected && connect_server() < 0)
 	return NULL;
 
@@ -971,31 +1067,18 @@ group_header *gh;
 	 case OK_GROUP:
 	    break;
 	 default:
-	    if (try_again) goto again; /* Handle nntp server timeouts */
 	    log_entry('N', "LISTGROUP response: %d", n);
 	    return NULL;
 	 case ERR_COMMAND:
 	    try_listgroup = 0;
-	    goto again;	/* error may have closed down server connection */
 	}
     }
     if (!try_listgroup) {
-	if (group_is_set == 0)
-	    switch (do_set_group()) {
-	     case -1:
-		return NULL;
-	     case 0:
-		goto again;
-	     case 1:
-		break;
-	    }
-
 	switch (n = ask_server("XHDR message-id %ld-%ld",
 		(long)gh->first_db_article, (long)gh->last_db_article)) {
 	 case OK_HEAD:
 	    break;
 	 default:
-	    if (try_again) goto again; /* Handle nntp server timeouts */
 	    log_entry('N', "XHDR response: %d", n);
 	    return NULL;
 	 case ERR_COMMAND:
@@ -1032,7 +1115,7 @@ group_header *gh;
 /*
  * nntp_get_article: get an article from the server.
  *
- * 	Returns a FILE pointer.
+ *	Returns a FILE pointer.
  *	If necessary the server's current group is set.
  *	The article (header and body) are copied into a file, so they
  *	are seekable (nn likes that).
@@ -1044,32 +1127,18 @@ static char *mode_cmd[] = {
     "BODY"
 };
 
-FILE *nntp_get_article(article, mode)
-article_number article;
-int mode;	/* 0 => whole article, 1 => head only, 2 => body only */
+FILE *
+nntp_get_article(article_number article, int mode)
+	/* mode: 0 => whole article, 1 => head only, 2 => body only */
 {
     FILE *tmp;
     static struct cache *cptr;
     int n;
-    
- again:
+
     if (!is_connected && connect_server() < 0) {
 	return NULL;
     }
 
-    /*
-     * Set the server group to the current group
-     */
-    if (group_is_set == 0)
-	switch (do_set_group()) {
-	 case -1:
-	    return NULL;
-	 case 0:
-	    goto again;
-	 case 1:
-	    break;
-	}
-
     /*
      * Search the cache for the requested article, and allocate a new
      * slot if necessary (if appending body, we already got it).
@@ -1108,15 +1177,14 @@ int mode;	/* 0 => whole article, 1 => head only, 2 => body only */
 	goto out;
 	
      case ERR_NOARTIG:
-	         /* Matt Heffron: ANUNEWS on VMS uses no such article error */
+		 /* Matt Heffron: ANUNEWS on VMS uses no such article error */
      case ERR_NOART:
 	return NULL;
 
      default:
-	if (try_again) goto again; /* Handle nntp server timeouts */
 	/* Matt Heffron: Which group? */
 	log_entry('N', "ARTICLE %ld response: %d (in Group %s)",
-                    (long)article, n, group_hd->group_name);
+		    (long)article, n, group_hd->group_name);
 	nntp_failed = 1;
 	return NULL;
     }
@@ -1125,16 +1193,17 @@ int mode;	/* 0 => whole article, 1 => head only, 2 => body only */
     return open_file(cptr->file_name, OPEN_READ|MUST_EXIST);
 
  err:
-    sys_error('N', "Cannot write temporary file %s", cptr->file_name);
+    /* sys_error('N', "Cannot write temporary file %s", cptr->file_name); */
+    nn_exitmsg(1, "Cannot write temporary file %s", cptr->file_name);
+    return NULL;	/* for lint, we can't actually get here */
 }
 
 /*
  *	Return local file name holding article
  */
 
-char *nntp_get_filename(art, gh)
-article_number art;
-group_header *gh;
+char *
+nntp_get_filename(article_number art, group_header *gh)
 {
     struct cache *cptr;
 
@@ -1153,13 +1222,8 @@ nntp_close_server()
     if (!is_connected)
 	return;
 
-    if (!nntp_failed) {			/* avoid infinite recursion */
-	int n;
-
-	n = ask_server("QUIT");
-	if (n != OK_GOODBYE)
-	    ;				/* WHAT NOW ??? */
-    }
+    if (!nntp_failed) 		/* avoid infinite recursion */
+	put_server("QUIT");
 
     (void) fclose(nntp_out);
     (void) fclose(nntp_in);
@@ -1167,16 +1231,19 @@ nntp_close_server()
     is_connected = 0;
 }
 
+
 /*
  * nntp_cleanup:  clean up after an nntp session.
  *
  *	Called from nn_exit().
  */
 
+void
 nntp_cleanup()
 {
     if (is_connected)
 	nntp_close_server();
+    is_connected = 0;
     clean_cache();
 }
 
@@ -1190,22 +1257,17 @@ nntp_cleanup()
 ** Sends the XOVER command and prepares to read the result.
 */
 struct novgroup *
-nntp_get_overview(gh, first, last)
-group_header *gh;
-article_number first, last;
+nntp_get_overview(group_header *gh, article_number first, article_number last)
 {
   int	n;
 
-again:
   if (!is_connected && connect_server() < 0) {
     return NULL;
   }
-  nntp_set_group(gh);
-  switch (do_set_group()) {
-  case -1:
-    return NULL;
-  case 0:
-    goto again;
+  switch (nntp_set_group(gh)) {
+	case -1:
+	case 0:
+	    return NULL;
   }
 
   n = ask_server("XOVER %d-%d", first, last);
@@ -1215,8 +1277,6 @@ again:
     return novstream(nntp_in);
 
   default:
-    if (try_again) 
-      goto again; /* Handle nntp server timeouts */
     log_entry('N', "XOVER response: %d", n);
     return NULL;
 
@@ -1232,12 +1292,10 @@ again:
  */
 
 FILE *
-nntp_fopen_list(cmd)
-char *cmd;
+nntp_fopen_list(char *cmd)
 {
   int	n;
 
-again:
   if (!is_connected && connect_server() < 0) 
     return NULL;
 
@@ -1250,7 +1308,6 @@ again:
     return nntp_in;
 
   default:
-    if (try_again) goto again;	/* Handle nntp server timeouts */
     log_entry('N', "`%s' response: %d", cmd, n);
     return NULL;
   }
@@ -1263,9 +1320,7 @@ again:
  * Returns NULL when NNTP sends the terminating "." to indicate EOF.
  */
 char *
-nntp_fgets(buf, bufsize)
-char *buf;
-int bufsize;
+nntp_fgets(char *buf, int bufsize)
 {
     char *cp;
     register int size;
@@ -1275,7 +1330,7 @@ int bufsize;
 
     cp = buf;
     if (*cp == '.') {
-        if (*++cp == '\0')
+	if (*++cp == '\0')
 	    return NULL;
     }
     cp[size]   = '\n';
@@ -1285,5 +1340,275 @@ int bufsize;
 
 #endif /* NOV */
 
+/* do all the AUTH stuff; used from 3 places that I've found where
+ * various news servers ask for authentication.  Given that we exit
+ * on failed auth, probably should just be void.
+ * OLSON: oops, loops on empty user input "forever". Should fix
+*/
+static void
+nntp_doauth(void)
+{
+    int len;
+    char line[NNTP_STRLEN], *nl, *pass = NULL;
+
+    strcpy(line, "authinfo user ");
+    len = strlen(line);
+
+    log_entry('N', "NNTP Authentication");
+
+    if(nntp_user) {
+	msg("NNTP authentication");
+	strcpy(line+len, nntp_user);
+    } else {
+	init_term(1);
+	clrdisp();
+	prompt("Authentication required; enter username: ");
+	(void)fgets(line+len, sizeof(line)-len, stdin);
+	if((nl=strpbrk(line, "\r\n")))
+	    *nl = '\0';
+	nntp_user = strdup(line+len);
+    }
+    put_server(line);
+    if((len=get_server(line, sizeof(line))) == OK_NEEDPASS) {
+	strcpy(line, "authinfo pass ");
+	len = strlen(line);
+
+	if(nntp_password)
+	    strcpy(line+len, nntp_password);
+	else {
+#ifdef USEGETPASSPHRASE
+	    pass = getpassphrase("Enter password: ");
+#else
+	    pass = getpass("Enter password: ");
+#endif
+	    strcat(line, pass);
+	}
+	put_server(line);
+	if((len=get_server(line, sizeof(line))) == OK_AUTH) {
+	    if(!nntp_password) {
+		nntp_password = strdup(pass);
+		clrdisp();
+		msg("Remembering password for reconnections");
+	    }
+	} else {
+	    nn_exitmsg(1, "Authentication failed (%d): %s", len, line);
+	}
+    }
+    need_auth = 1;
+}
+
+
+/* ZZZZZ */
+
+/*
+ * The rest of this is cribbed (and modified) from the old mini-inews.
+ *
+ * Simply accept input on stdin (or via a named file) and dump this
+ * to the server; add a From: and Path: line if missing in the original.
+ * Print meaningful errors from the server.
+ * Limit .signature files to MAX_SIGNATURE lines.
+ *
+ * Original by Steven Grady <grady@ucbvax.Berkeley.EDU>, with thanks from
+ * Phil Lapsley <phil@ucbvax.berkeley.edu>
+ */
+
+/*
+ * gen_frompath -- generate From: and Path: lines, in the form
+ *
+ *	From: Full Name <user@host.domain>
+ *	Path: host!user
+ *
+ * This routine should only be called if the message doesn't have
+ * a From: line in it.
+ */
+
+static void
+gen_frompath()
+{
+	struct	passwd *passwd;
+
+	passwd = getpwuid(getuid());
+
+	fprintf(nntp_out, "From: ");
+	fprintf(nntp_out, "%s ", full_name());
+
+	fprintf(nntp_out, "<%s@%s>\r\n",
+		passwd->pw_name,
+		domain);
+
+#ifdef HIDDENNET
+	/* Only the login name - nntp server will add uucp name */
+	fprintf(nntp_out, "Path: %s\r\n", passwd->pw_name);
+#else /* HIDDENNET */
+	fprintf(nntp_out, "Path: %s!%s\r\n", host_name, passwd->pw_name);
+#endif /* HIDDENNET */
+}
+
+
+/*
+ * lower -- convert a character to lower case, if it's
+ *	upper case.
+ *
+ *	Parameters:	"c" is the character to be
+ *			converted.
+ *
+ *	Returns:	"c" if the character is not
+ *			upper case, otherwise the lower
+ *			case eqivalent of "c".
+ *
+ *	Side effects:	None.
+ */
+
+static char
+lower(register char c)
+{
+	if (isascii(c) && isupper(c))
+		c = c - 'A' + 'a';
+	return(c);
+}
+
+/*
+ * strneql -- determine if two strings are equal in the first n
+ * characters, ignoring case.
+ *
+ *	Parameters:	"a" and "b" are the pointers
+ *			to characters to be compared.
+ *			"n" is the number of characters to compare.
+ *
+ *	Returns:	1 if the strings are equal, 0 otherwise.
+ *
+ *	Side effects:	None.
+ */
+
+static int
+strneql(register char *a, register char *b, int n)
+{
+	while (n && lower(*a) == lower(*b)) {
+		if (*a == '\0')
+			return (1);
+		a++;
+		b++;
+		n--;
+	}
+	if (n)
+		return (0);
+	else
+		return (1);
+}
+
+
+/*
+ * valid_header -- determine if a line is a valid header line
+ *
+ *	Parameters:	"h" is the header line to be checked.
+ *
+ *	Returns:	1 if valid, 0 otherwise
+ *
+ *	Side Effects:	none
+ *
+ */
+
+static int
+valid_header(register char *h)
+{
+  char *colon, *space;
+
+  /*
+   * blank or tab in first position implies this is a continuation header
+   */
+  if (h[0] == ' ' || h[0] == '\t')
+    return (1);
+
+  /*
+   * just check for initial letter, colon, and space to make
+   * sure we discard only invalid headers
+   */
+  colon = index(h, ':');
+  space = index(h, ' ');
+  if (isalpha(h[0]) && colon && space == colon + 1)
+    return (1);
+
+  /*
+   * anything else is a bad header -- it should be ignored
+   */
+  return (0);
+}
+
+int
+nntp_post(char *temp_file)
+{
+	import char	delayed_msg[];
+	int	n, response;
+	FILE	*in = fopen(temp_file, "r");
+	char	s[4*NNTP_STRLEN];
+	int	seen_fromline, in_header, seen_header;
+	register char	*cp;
+
+	if(!in) {
+	    sprintf(delayed_msg,"Posting failed because we couldn't re-open file %s.", temp_file);
+	    return 1;
+	}
+
+	if (!is_connected && connect_server() < 0)
+	    return 1;
+
+	switch (n=ask_server("POST")) {
+	  case CONT_POST:
+	    break;
+	  default:
+	    sprintf(delayed_msg,"Request to post failed with error %d, %s", n, ask_reply);
+	    fclose(in);
+	    return 1;
+	}
+
+	in_header = 1;
+	seen_header = 0;
+	seen_fromline = 0;
+
+	while (fgets(s, sizeof(s), in) != NULL) {
+		if ((cp = strchr(s, '\n')))
+			*cp = '\0';
+		if (s[0] == '.')    /* Single . is eof, so put in extra one */
+			(void) fputc('.', nntp_out);
+		if (in_header && strneql(s, "From:", sizeof("From:")-1)) {
+			seen_header = 1;
+			seen_fromline = 1;
+		}
+		if (in_header && s[0] == '\0') {
+			if (seen_header) {
+				in_header = 0;
+				if (!seen_fromline)
+					gen_frompath();
+				fprintf(nntp_out, "User-Agent: nn/%s.%s\r\n", RELEASE, PATCHLEVEL);
+			} else {
+				continue;
+			}
+		} else if (in_header) {
+			if (valid_header(s))
+				seen_header = 1;
+			else
+				continue;
+		}
+		fprintf(nntp_out, "%s\r\n", s);
+	}
+
+	fprintf(nntp_out, ".\r\n");
+	(void) fflush(nntp_out);
+	response = get_server(ask_reply, sizeof(ask_reply));
+	switch (response) {
+	  case OK_POSTED:
+	    break;
+	  case ERR_POSTFAIL:
+	    msg("Article not accepted by server; not posted.");
+	    user_delay(2);
+	    return 1;
+	  default:
+	    msg("Remote error: %s", ask_reply);
+	    user_delay(2);
+	    return 1;
+	}
+	return 0;
+}
+
 #endif /* NNTP */
 
