@@ -14,11 +14,6 @@
 #ifdef RESIZING
 #include <sys/ioctl.h>			/* for TIOCGWINSZ */
 
-#ifdef SYSV_RESIZING
-#include <sys/stream.h>
-#include <sys/ptem.h>
-#endif	/* SYSV_RESIZING */
-
 extern int s_resized;
 #endif	/* RESIZING */
 
@@ -47,9 +42,16 @@ extern int s_resized;
 #  include <termio.h>
 # endif	/* USE_TERMCAP */
 #else
+#ifndef __FreeBSD__
 # include <sgtty.h>
+#endif	/* __FreeBSD__ */
 #endif
 
+#ifdef SYSV_RESIZING
+#include <sys/stream.h>
+#include <sys/ptem.h>
+#endif	/* SYSV_RESIZING */
+
 /* SYSV curses.h clash */
 #undef raw
 
@@ -61,20 +63,18 @@ extern int s_resized;
 
 /* term.c */
 
-#ifdef __STDC__
-#ifdef APOLLO_DOMAIN_OS
-static int outc __APROTO((int c));
+#ifdef HAVE_TERMIOS_H
+static int      outc(int c);
 #else
-static int outc __APROTO((char c));
-#endif	/* APOLLO_DOMAIN_OS */
-#else
-static int outc __APROTO(());
-#endif	/* __STDC__ */
+static int      outc(char c);
+#endif
+
 static void set_term_speed __APROTO((register unsigned long sp));
 static void raw_not_ok_error __APROTO((void));
 static sig_type rd_timeout __APROTO((int n));
 static int read_char_kbd __APROTO((int tmo));
 static void unread_char __APROTO((int c));
+static int nnstandout(int);
 
 static sig_type catch_winch();
 
@@ -172,31 +172,19 @@ static char key_mouse_u1[64] = "\33[M#";
  */	
 #define DISCARD_REMAINDER(x,p) ((x) & ~((p)-1))
 
-#ifdef __STDC__
+#ifdef HAVE_TERMIOS_H
 static int
-#ifdef APOLLO_DOMAIN_OS
 outc(int c)
 #else
-outc(char c)
-#endif	/* APOLLO_DOMAIN_OS */
-#else
 static int
-outc(c) char c;
-#endif	/* __STDC__ */
+outc(char c)
+#endif
 {
     putchar(c);
     return 0;			/* XXX What is it supposed to return? */
 }
 
-
-#ifdef BROKEN_TPUTS
-/* tputs is broken on UNISYS I've been told */
-#define putpc(str, cnt)	tputs(str, 0, outc)
-#else
-#define putpc(str, cnt)	tputs(str, cnt, outc)
-#endif
-
-#undef standout
+#define putpc(str, cnt) tputs(str, cnt, outc)
 
 int  Lines, Columns;	/* screen size */
 int  cookie_size;	/* size of magic cookie */
@@ -239,7 +227,7 @@ static struct termio norm_tty, raw_tty;
 #define	EraseC	((key_type) norm_tty.c_cc[VERASE])
 #define KillC	((key_type) norm_tty.c_cc[VKILL])
 #ifdef HAVE_TERMIOS_H
-#define SuspC   ((key_type) norm_tty.c_cc[VSUSP])
+#define SuspC	((key_type) norm_tty.c_cc[VSUSP])
 #else
 #define SuspC	((key_type) CONTROL_('Z'))	/* norm_tty.c_cc[SWTCH] */
 #endif
@@ -254,7 +242,7 @@ static struct tchars norm_chars;
 #define KillC	((key_type) norm_tty.sg_kill)
 
 #ifdef TIOCGLTC
-static struct 	ltchars spec_chars;
+static struct	ltchars spec_chars;
 #define SuspC	((key_type) spec_chars.t_suspc)
 #else
 #define	SuspC	((key_type) CONTROL_('Z'))
@@ -281,7 +269,7 @@ char *cap, *buf;
 		   term_name, cap);
 }
 
-#endif  /* USE_TERMCAP */
+#endif	/* USE_TERMCAP */
 
 /*
  * timeout in n/10 seconds via SIGALRM
@@ -435,7 +423,7 @@ static unsigned sp_table[] = {
     0,	0
 };
 
-static  void set_term_speed(sp)
+static	void set_term_speed(sp)
 register unsigned long sp;
 {
     register unsigned *tp;
@@ -645,7 +633,7 @@ int full;
 #ifdef SV_INTERRUPT
     siginterrupt(SIGALRM, 1);		/* make read from tty interruptable */
 #endif	/* SV_INTERRUPT */
-#endif  /* HAVE_TERMIOS_H */
+#endif	/* HAVE_TERMIOS_H */
 #endif	/* HAVE_TERMIO_H */
 
     erase_key = EraseC;
@@ -725,7 +713,7 @@ int c, l;
     if (!(c | l))
 	putp(cursor_home);
     else
-        putp(tgoto(cursor_address, c, l));
+	putp(tgoto(cursor_address, c, l));
     curxy_c = c; curxy_l = l;
 }
 
@@ -747,7 +735,7 @@ clrdisp()
     msg_ptr = msg_stack;
 }
 
-void     clrline_noflush();
+void	 clrline_noflush();
 
 void
 clrline()
@@ -783,14 +771,14 @@ clrline_noflush()
 #ifdef TERM_DEBUG
 	if (term_debug) fprintf(stderr, " ignored\n");
 #endif
-        return;
+	return;
     }
 
     if (HAS_CAP(clr_eol)) {
 #ifdef TERM_DEBUG
-        if (term_debug) fprintf(stderr, "\n");
+	if (term_debug) fprintf(stderr, "\n");
 #endif
-        putp(clr_eol);
+	putp(clr_eol);
     } else {
 	oldxy_c = curxy_c; oldxy_l = curxy_l;
 
@@ -808,7 +796,7 @@ clrline_noflush()
 #ifdef TERM_DEBUG
 	if (term_debug && *term_debug) {
 	    while (spcnt--) tputc(*term_debug);
-  	    spcnt = 0;
+	    spcnt = 0;
 	}
 #endif
 	/* clear out line */
@@ -848,17 +836,17 @@ clrpage()
     /* clear to end of screen */
 
     if (curxy_c == 0) {
-        if (HAS_CAP(clr_eos)) {
+	if (HAS_CAP(clr_eos)) {
 	    putpc(clr_eos, Lines - curxy_l);
 	    for (i = curxy_l; i < Lines; i++) nonsp[i] = 0;
-        } else if (curxy_l == 0) {
+	} else if (curxy_l == 0) {
 	    putpc(clear_screen, Lines);
 	    for (i = 0; i < Lines; i++) nonsp[i] = 0;
-        } else {
+	} else {
 	    for (i = curxy_l; i < Lines; i++) {
 		if (nonsp[i] != 0) {
 		    gotoxy(0, i);
-    	            clrline_noflush();
+		    clrline_noflush();
 		}
 	    }
 	}
@@ -873,31 +861,28 @@ clrpage()
 
 static char buf[512];
 
-/*VARARGS*/
-void tprintf(va_alist)
-va_dcl
+static void
+tvprintf(char *fmt, va_list ap)
 {
-    use_vararg;
-
-    start_vararg;
-    tvprintf(va_args1toN);
-    end_vararg;
-}
-
-void tvprintf(va_tail)
-va_tdcl
-{
-    char *fmt, *str;
-
-    fmt = va_arg1(char *);
-    vsprintf(buf, fmt, va_args2toN);
+    char *str;
 
+    vsprintf(buf, fmt, ap);
     for (str = buf; *str; str++)
-        tputc(*str);
+	tputc(*str);
 }
 
-void tputc(c)
-int c;
+void
+tprintf(char *fmt,...)
+{
+    va_list	ap;
+
+    va_start(ap, fmt);
+    tvprintf(fmt, ap);
+    va_end(ap);
+}
+
+void
+tputc(int c)
 {
     int i;
 
@@ -905,6 +890,110 @@ int c;
     if (term_debug) {
 	fprintf (stderr, "tputc %d %d [%d] ", curxy_c, curxy_l, curxy_nonsp);
 	if (c < ' ' || c > '~')
+	    switch (c) {
+		case 0:
+		    fprintf(stderr, "'NUL'");
+		    break;
+		case 1:
+		    fprintf(stderr, "'SOH'");
+		    break;
+		case 2:
+		    fprintf(stderr, "'STX'");
+		    break;
+		case 3:
+		    fprintf(stderr, "'ETX'");
+		    break;
+		case 4:
+		    fprintf(stderr, "'EOT'");
+		    break;
+		case 5:
+		    fprintf(stderr, "'ENQ'");
+		    break;
+		case 6:
+		    fprintf(stderr, "'ACK'");
+		    break;
+		case 7:
+		    fprintf(stderr, "'BEL'");
+		    break;
+		case 8:
+		    fprintf(stderr, "'BS'");
+		    break;
+		case 9:
+		    fprintf(stderr, "'HT'");
+		    break;
+		case 10:
+		    fprintf(stderr, "'NL'");
+		    break;
+		case 11:
+		    fprintf(stderr, "'VT'");
+		    break;
+		case 12:
+		    fprintf(stderr, "'NP'");
+		    break;
+		case 13:
+		    fprintf(stderr, "'CR'");
+		    break;
+		case 14:
+		    fprintf(stderr, "'SP'");
+		    break;
+		case 15:
+		    fprintf(stderr, "'SI'");
+		    break;
+		case 16:
+		    fprintf(stderr, "'DLE'");
+		    break;
+		case 17:
+		    fprintf(stderr, "'DC1'");
+		    break;
+		case 18:
+		    fprintf(stderr, "'DC2'");
+		    break;
+		case 19:
+		    fprintf(stderr, "'DC3'");
+		    break;
+		case 20:
+		    fprintf(stderr, "'DC4'");
+		    break;
+		case 21:
+		    fprintf(stderr, "'NAK'");
+		    break;
+		case 22:
+		    fprintf(stderr, "'SYN'");
+		    break;
+		case 23:
+		    fprintf(stderr, "'ETB'");
+		    break;
+		case 24:
+		    fprintf(stderr, "'CAN'");
+		    break;
+		case 25:
+		    fprintf(stderr, "'EM'");
+		    break;
+		case 26:
+		    fprintf(stderr, "'SUB'");
+		    break;
+		case 27:
+		    fprintf(stderr, "'ESC'");
+		    break;
+		case 28:
+		    fprintf(stderr, "'FS'");
+		    break;
+		case 29:
+		    fprintf(stderr, "'GS'");
+		    break;
+		case 30:
+		    fprintf(stderr, "'RS'");
+		    break;
+		case 31:
+		    fprintf(stderr, "'US'");
+		    break;
+		case 32:
+		    fprintf(stderr, "'SP'");
+		    break;
+		default:
+		    fprintf(stderr, "out of range: ???");
+		    break;
+	    }
 	else
 	    fprintf(stderr, "'%c'", c);
     }
@@ -918,16 +1007,17 @@ int c;
 
     case '\n':
 	curxy_c = 0;
-	if (curxy_l >= 0) curxy_l++;
+	if (curxy_l >= 0)
+	    curxy_l++;
 	break;
 
     case '\r':
 	curxy_c = 0;
-    	break;
+	break;
 
     case '\t':
 	curxy_c = DISCARD_REMAINDER(curxy_c + 8, 8);
- 	break;
+	break;
 
     case '\b':
 	curxy_c--;
@@ -955,17 +1045,20 @@ int c;
     /* account for right margin */
 
     if (curxy_c == Columns) {
+
 #ifdef TERM_DEBUG
-	if (term_debug) fprintf(stderr, " margin");
+	if (term_debug)
+	    fprintf(stderr, " margin");
 #endif
 	if (auto_right_margin) {
 	    if (eat_newline_glitch) {
 		putchar(CR);
-                just_sent_cr = 1;
+		just_sent_cr = 1;
 		curxy_c = 0;
 	    } else {
 		curxy_c = 0;
-		if (curxy_l >= 0) curxy_l++;
+		if (curxy_l >= 0)
+		    curxy_l++;
 	    }
 	} else
 	    curxy_c--;
@@ -974,30 +1067,34 @@ int c;
     /* account for vertical scroll */
 
     if (curxy_l == Lines) {
+
 #ifdef TERM_DEBUG
-	if (term_debug) fprintf(stderr, " scroll");
+	if (term_debug)
+	    fprintf(stderr, " scroll");
 #endif
+
 	for (i = 1; i < Lines; i++)
 	    nonsp[i-1] = nonsp[i];
 	nonsp[--curxy_l] = 0;
     }
 
 #ifdef TERM_DEBUG
-    if (term_debug) fprintf(stderr, "\n");
+    if (term_debug)
+	fprintf(stderr, "\n");
 #endif
 }
 
 static char so_buf[512], *so_p;
 static int so_c, so_l, so_b, so_active = 0;
 
-int so_gotoxy(c, l, blank)
-int c, l, blank;
+int
+so_gotoxy(int c, int l, int blank)
 {
     if (!STANDOUT && c >= 0) {
-	if (c>= 0 && l >= 0) gotoxy(c, l);
+	if (c>= 0 && l >= 0)
+	    gotoxy(c, l);
 	return 0;
     }
-
     so_active++;
     so_c = c;
     so_l = l;
@@ -1008,38 +1105,37 @@ int c, l, blank;
     return 1;	/* not really true if not standout & c < 0 */
 }
 
-/*VARARGS*/
-void so_printf(va_alist)
-va_dcl
+static void
+so_vprintf(char *fmt, va_list ap)
 {
-    use_vararg;
-
-    start_vararg;
-    so_vprintf(va_args1toN);
-    end_vararg;
-}
-
-void so_vprintf(va_tail)
-va_tdcl
-{
-    char *fmt;
-
     if (!so_active) {
-	if (ceol_standout_glitch) highlight(0); /* xxx why? */
-	tvprintf(va_args1toN);
+	if (ceol_standout_glitch)
+	    highlight(0); /* xxx why? */
+	tvprintf(fmt, ap);
 	return;
     }
-
-    fmt = va_arg1(char *);
-    vsprintf(so_p, fmt, va_args2toN);
-    while (*so_p) so_p++;
+    vsprintf(so_p, fmt, ap);
+    while (*so_p)
+	so_p++;
 }
 
-void so_end()
+void
+so_printf(char *fmt,...)
+{
+    va_list	ap;
+
+    va_start(ap, fmt);
+    so_vprintf(fmt, ap);
+    va_end(ap);
+}
+
+void
+so_end()
 {
     int len;
 
-    if (!so_active) return;
+    if (!so_active)
+	return;
 
     if (so_l >= 0) {
 
@@ -1047,93 +1143,98 @@ void so_end()
 
 	if (so_c < 0)
 	    so_c = Columns - len - 2;
-	if (so_c < 0) so_c = 0;
+	if (so_c < 0)
+	    so_c = 0;
 
 	if (len + so_c >= Columns) {
 	    len = Columns - so_c - two_cookies;
 	    so_buf[len] = NUL;
 	}
-
 	if (cookie_size) {
 	    gotoxy(so_c + len - cookie_size, so_l);
-	    standout(0);
+	    nnstandout(0);
 	}
-
 	gotoxy(so_c, so_l);
 
     }
-
-    if ((so_b & 1) && (!STANDOUT || !cookie_size)) tputc(SP);
+    if ((so_b & 1) && (!STANDOUT || !cookie_size))
+	tputc(SP);
 
     if (STANDOUT) {
-	if (ceol_standout_glitch) clrline();
-	standout(1);
+	if (ceol_standout_glitch)
+	    clrline();
+	nnstandout(1);
     }
-
     tprintf("%s", so_buf);
 
-    if (STANDOUT) standout(0);
+    if (STANDOUT)
+	nnstandout(0);
 
-    if ((so_b & 2) && (!STANDOUT || !cookie_size)) tputc(SP);
+    if ((so_b & 2) && (!STANDOUT || !cookie_size))
+	tputc(SP);
 
     so_active = 0;
 }
 
 
-/*VARARGS*/
-void so_printxy(va_alist)
-va_dcl
+void
+so_printxy(int c, int l, char *fmt,...)
 {
-    int c, l;
-    use_vararg;
-
-    start_vararg;
-
-    c = va_arg1(int);
-    l = va_arg2(int);
+    va_list	ap;
 
+    va_start(ap, fmt);
     so_gotoxy(c, l, 0);
-    so_vprintf(va_args3toN);
+    so_vprintf(fmt, ap);
     so_end();
-
-    end_vararg;
+    va_end(ap);
 }
 
     
-int underline(on)
-int on;
+int
+underline(int on)
 {
-    if (cookie_size) return 0;
+    if (cookie_size)
+	return 0;
+
 #ifdef TERM_DEBUG
     if (term_debug)
 	fprintf(stderr, "underline %d %d [%d] %d", curxy_c, curxy_l, curxy_nonsp, on);
 #endif
-    if (! HAS_CAP(enter_underline_mode)) return 0;
+
+    if (! HAS_CAP(enter_underline_mode))
+	return 0;
     putp(on ? enter_underline_mode : exit_underline_mode);
     return 1;
 }
 
-int highlight(on)
-int on;
+int
+highlight(int on)
 {
-    if (cookie_size) return 0;
+    if (cookie_size)
+	return 0;
+
 #ifdef TERM_DEBUG
     if (term_debug)
 	fprintf(stderr, "highlight %d %d [%d] %d", curxy_c, curxy_l, curxy_nonsp, on);
 #endif
-    if (! HAS_CAP(enter_standout_mode)) return 0;
+
+    if (! HAS_CAP(enter_standout_mode))
+	return 0;
     putp(on ? enter_standout_mode : exit_standout_mode);
     return 1;
 }
 
-int shadeline(on)
-int on;
+int
+shadeline(int on)
 {
-    if (cookie_size) return 0;
+    if (cookie_size)
+	return 0;
+
 #ifdef TERM_DEBUG
     if (term_debug)
 	fprintf(stderr, "shadeline %d %d [%d] %d", curxy_c, curxy_l, curxy_nonsp, on);
 #endif
+
     if (shade_on_attr && shade_off_attr) {
 	putp(on ? shade_on_attr : shade_off_attr);
 	return 1;
@@ -1141,22 +1242,27 @@ int on;
 	return underline(1);
 }
     
-int standout(on)
-int on;
+static int
+nnstandout(int on)
 {
+
 #ifdef TERM_DEBUG
     if (term_debug)
 	fprintf(stderr, "standout %d %d [%d] %d", curxy_c, curxy_l, curxy_nonsp, on);
 #endif
-    if (! HAS_CAP(enter_standout_mode)) return 0;
+
+    if (! HAS_CAP(enter_standout_mode))
+	return 0;
     putp(on ? enter_standout_mode : exit_standout_mode);
     curxy_c += cookie_size;
     if (curxy_l >= 0 && curxy_c > nonsp[curxy_l])
 	nonsp[curxy_l] = curxy_c;
+
 #ifdef TERM_DEBUG
     if (term_debug)
 	fprintf(stderr, " -> %d %d [%d]\n", curxy_c, curxy_l, curxy_nonsp);
 #endif
+
     return 1;
 }
 
@@ -1167,6 +1273,7 @@ static int is_raw = 0;
 #define RAW_MODE_ON    ioctl(0, TCSETAW, &raw_tty)
 #define RAW_MODE_OFF   ioctl(0, TCSETAW, &norm_tty)
 #else
+
 #ifdef HAVE_TERMIOS_H
 #define RAW_MODE_ON    tcsetattr(0, TCSADRAIN, &raw_tty)
 #define RAW_MODE_OFF   tcsetattr(0, TCSADRAIN, &norm_tty)
@@ -1174,6 +1281,7 @@ static int is_raw = 0;
 #define RAW_MODE_ON    ioctl(0, TIOCSETP, &raw_tty)
 #define RAW_MODE_OFF   ioctl(0, TIOCSETP, &norm_tty)
 #endif	/* HAVE_TERMIOS_H */
+
 #endif	/* HAVE_TERMIO_H */
 
 void
@@ -1193,15 +1301,18 @@ visual_on()
 {
     BATCH_CHECK_V;
 
-    if (terminal_speed == 0) return;
+    if (terminal_speed == 0)
+	return;
 
 #ifdef TERM_DEBUG
     if (term_debug) fprintf(stderr, "visual_on\n");
 #endif
 
     if (!is_visual) {
-	if (HAS_CAP(enter_ca_mode)) putp(enter_ca_mode);
-	if (appl_keypad_mode) putp(keypad_xmit);
+	if (HAS_CAP(enter_ca_mode))
+	    putp(enter_ca_mode);
+	if (appl_keypad_mode)
+	    putp(keypad_xmit);
 	if (mouse_state)
 	    xterm_mouse_on();
 	fl;
@@ -1209,19 +1320,24 @@ visual_on()
     is_visual = 1;
 }
 
-int visual_off()
+int
+visual_off()
 {
     int was_raw = is_raw;
 
-    if (terminal_speed == 0) return 0;
+    if (terminal_speed == 0)
+	return 0;
 
 #ifdef TERM_DEBUG
-    if (term_debug) fprintf(stderr, "visual_off\n");
+    if (term_debug)
+	fprintf(stderr, "visual_off\n");
 #endif
 
     if (is_visual) {
-	if (appl_keypad_mode) putp(keypad_local);
-	if (HAS_CAP(exit_ca_mode)) putp(exit_ca_mode);
+	if (appl_keypad_mode)
+	    putp(keypad_local);
+	if (HAS_CAP(exit_ca_mode))
+	    putp(exit_ca_mode);
 	if (mouse_state)
 	    xterm_mouse_off();
 	fl;
@@ -1274,19 +1390,22 @@ nn_raw()
     RAW_CHECK_V;
 
     if (!flow_control) {
-	if (!must_set_raw) return;
+	if (!must_set_raw)
+	    return;
 	must_set_raw = 0;
     }
-
-    if (is_raw) return;
+    if (is_raw)
+	return;
 
     RAW_MODE_ON;
     is_raw++;
 }
 
-int no_raw()
+int
+no_raw()
 {
-    if (!flow_control) return 0;
+    if (!flow_control)
+	return 0;
 
     if (!is_raw) return 0;
 
@@ -1299,7 +1418,8 @@ int no_raw()
     return 1;
 }
 
-int unset_raw()
+int
+unset_raw()
 {
     int was_raw = is_raw;
 
@@ -1308,7 +1428,6 @@ int unset_raw()
 	RAW_MODE_OFF;
 	is_raw = 0;
     }
-
     if (!flow_control)
 	must_set_raw = 1;
     return was_raw;
@@ -1319,6 +1438,7 @@ int unset_raw()
 #ifndef KEY_BURST
 #define KEY_BURST 32
 #endif	/* KEY_BURST */
+
 #define RD_PUSHBACK 10
 static char rd_buffer[KEY_BURST+RD_PUSHBACK];	/* Holds stuff from read */
 static char *rd_ptr;
@@ -1401,7 +1521,7 @@ flush_input()
 #else
     ioctl(0, TIOCFLUSH, 0);
 #endif	/* FREAD */
-#endif  /* HAVE_TERMIOS_H */
+#endif	/* HAVE_TERMIOS_H */
 #endif	/* HAVE_TERMIO_H */
     rd_count = 0;
 }
@@ -1483,7 +1603,7 @@ int get_c()
 	if (*(mk->cur_key) == NUL) {
 	    c = mk->code;
 
-            /* xterm mouse specific code, translate cursor position
+	    /* xterm mouse specific code, translate cursor position
 	       into static variables, synthesize key up events */
 	    if ((c >= K_m_d1) && (c <= K_m_u1)) {
 		mouse_x = read_char_kbd(key_cnt) - '!';
@@ -1560,7 +1680,7 @@ int get_c()
  *	break_chars	return immediately if one of these characters
  *			is entered as the first character.
  *
- *	completion 	is a function that will fill the buffer with a value
+ *	completion	is a function that will fill the buffer with a value
  *			see the group_completion and file_completion routines
  *			for examples.
  */
@@ -1838,7 +1958,7 @@ int must_answer;
 	return 1;
      case 3:
 	do_macro_processing = 0;
-    	in_macro++;
+	in_macro++;
 	break;
     }
     fl;
@@ -2003,43 +2123,26 @@ chain:
 }
 
 
-/*VARARGS*/
-void nn_exitmsg(va_alist)
-va_dcl
+void
+nn_exitmsg(int n, char *fmt,...)
 {
-    char *fmt;
-    int n;
-    use_vararg;
+    va_list	ap;
 
     if (terminal_speed != 0) {
 	clrdisp();
 	visual_off();
     }
-
-    start_vararg;
-    n = va_arg1(int);
-    fmt = va_arg2(char *);
-    vprintf(fmt, va_args3toN);
+    va_start(ap, fmt);
+    vprintf(fmt, ap);
     putchar(NL);
-    end_vararg;
+    va_end(ap);
 
     nn_exit(n);
     /*NOTREACHED*/
 }
 
-/*VARARGS*/
-void msg(va_alist)
-va_dcl
-{
-    use_vararg;
-
-    start_vararg;
-    vmsg(va_args1toN);
-    end_vararg;
-}
-
-void push_msg(str)
-char *str;
+void
+push_msg(char *str)
 {
     register struct msg_list *mp, *newmsg;
     static int slots = 0;
@@ -2066,20 +2169,17 @@ char *str;
     msg_ptr = msg_stack;
 }
     
-void vmsg(va_tail)
-va_tdcl
+void
+vmsg(char *fmt, va_list ap)
 {
-    char *errmsg, *fmt;
-    
-    fmt = va_arg1(char *);
+    char *errmsg;
 
     if (fmt) {
 	char lbuf[512];
 	
-	vsprintf(lbuf, fmt, va_args2toN);
+	vsprintf(lbuf, fmt, ap);
 	push_msg(lbuf);
     }
-    
     if (msg_ptr) {
 	errmsg = msg_ptr->buf;
 	msg_ptr = msg_ptr->prev;
@@ -2093,20 +2193,28 @@ va_tdcl
 	fl;
 	return;
     }
-    
     gotoxy(0, Lines-1);
     tprintf("%s", errmsg);
     clrline();
     any_message = 1;
 
     if (prompt_line != Lines-1)
-        gotoxy(prompt_length, prompt_line);
+	gotoxy(prompt_length, prompt_line);
     fl;
 }
 
 void
-clrmsg(col)
-int col;
+msg(char *fmt,...)
+{
+    va_list	ap;
+
+    va_start(ap, fmt);
+    vmsg(fmt, ap);
+    va_end(ap);
+}
+
+void
+clrmsg(int col)
 {
     BATCH_CHECK_V;
 
@@ -2119,23 +2227,18 @@ int col;
 }
 
 
-/*VARARGS*/
 void
-prompt(va_alist)
-va_dcl
+prompt(char *fmt,...)
 {
+    va_list	ap;
     register char *cp;
     int stand_on;
-    char *fmt;
     static char cur_p[FILENAME];
     static char saved_p[FILENAME];
-    use_vararg;
 
     BATCH_CHECK_V;
 
-    start_vararg;
-
-    fmt = va_arg1(char *);
+    va_start(ap, fmt);
 
     if (fmt == P_VERSION) {
 	gotoxy(0, prompt_line + 1);
@@ -2147,12 +2250,10 @@ va_dcl
 	    gotoxy(prompt_length, prompt_line);
 	goto out;
     }
-
     if (fmt == P_SAVE) {
 	strcpy(saved_p, cur_p);
 	goto out;
     }
-
     if (fmt == P_RESTORE)
 	strcpy(cur_p, saved_p);
 
@@ -2163,9 +2264,8 @@ va_dcl
 	clrline();
 	goto out;
     }
-
     if (fmt != P_REDRAW && fmt != P_RESTORE)
-	vsprintf(cur_p, fmt, va_args2toN);
+	vsprintf(cur_p, fmt, ap);
 
     tputc(CR);
 
@@ -2174,18 +2274,21 @@ va_dcl
 	    if (cp[1] != '\1') {
 		if (STANDOUT) {
 		    stand_on = !stand_on;
-		    standout(stand_on);
+		    nnstandout(stand_on);
 		    prompt_length += cookie_size;
 		}
 		continue;
 	    }
 	    cp++;
-	} else
-	if (*cp == '\2') {
+	} else if (*cp == '\2') {
 	    time_t t;
 	    char   *timestr;
 
+#ifdef STATISTICS
 	    t = tick_usage();
+#else
+	    t = cur_time();
+#endif
 
 	    if (show_current_time) {
 		timestr = ctime(&t) + 11;
@@ -2194,39 +2297,36 @@ va_dcl
 		tprintf("-- %s ", timestr);
 		prompt_length += 9;
 	    }
-
 	    if (unread_mail(t)) {
 		tprintf("Mail ");
 		prompt_length += 5;
 	    }
-
 	    continue;
 	}
-
 	tputc(*cp);
 	prompt_length ++;
     }
     if (stand_on) {
-	standout(0);
+	nnstandout(0);
 	prompt_length += cookie_size;
     }
-
     clrline();
 
     if (fmt == P_RESTORE)
 	restore_xy();
+
 #if notdef
     else
 	curxy_c = -1;
 #endif
 
  out:
-    end_vararg;
+    va_end(ap);
 }
 
 
-int any_key(line)
-int line;
+int
+any_key(int line)
 {
     int was_raw, c, dmp;
 
